#ifndef _Organism
#define _Organism

#include <iostream>
#include <list>
#include <vector>
#include "Neuron.h"
#include "Functions.h"
#include "IDCounters.h"
#include "Definitions.h"
#include "Innovation.h"
#include "Neat.h"
#include <cstring>

using namespace std;

/* A NETWORK/Organism is a LIST of input NODEs and a LIST of output NODEs           
   The point of the network is to define a single entity which can evolve
   or learn on its own, even though it may be part of a larger framework */

class Organism {

public:

	// structural members (genotype)
	list<Neuron*> inputs; //Nodes that input into the network
	list<Neuron*> outputs; //Values output by the network
	list<Neuron*> hidden_nodes; // all the hidden neurons in the net (empty for each one at the beginning)
	int id;
	// phenotype:
	double fitness,orig_fitness; //A measure of fitness for the Organism
	int expected_offspring; //Number of children this Organism may have
	int generation; //Tells which generation this Organism is from
	bool eliminate; // true if too bad or too old to survive
	bool winner; // tells if this genome solve the problem
	int nb_links; // number of links (enable or not); useful for mutation and statistics
	
	// innovations experienced by this organism
	list<Link*> link_innovations;
	list<LinkAndNode*> node_innovations;

	Organism () {
		id=cur_org_id++;
		generation=0;
		eliminate=false;
		winner=false;
		nb_links=0;
	}

	//Constructor which spawns off an input file, with weights random or given by user
	//This constructor assumes that some routine has already read in GENOMESTART
	/* file format:
	 * genomestart <random_weights> // random_weights is a boolean telling if the link's weights are given
	 * incnode <id> <functype>
	 * incnode <id> <functype>
	 * ..etc
	 * [ optional:
	 * midnode <id> <functype> <incoming_id1> [<weight1>] ... <incoming_idn> [<weightn>] ; 
	 * midnode <id> <functype> <incoming_id1> [<weight1>] ... <incoming_idn> [<weightn>] ; 
	 * ..etc ]
	 * outnode <id> <functype> <incoming_id1> <innov_num1> [<weight1>] ... <incoming_idn> <innov_numn> [<weightn>] ;
	 * outnode <id> <functype> <incoming_id1> <innov_num1> [<weightn>] ... <incoming_idn> <innov_numn> [<weightn>] ;
	 * ..etc
	 * [ optional, depending if hidden_nodes empty or not:
	 * NOTE: we could also infer them, giving arbitrary innovation numbers,
	 * but as the mutations depend on innovation numbers it could be interesting to enter them:
	 * link_innovations <id1> <id2> <innov_num> ; <id1> <id2> <innov_num> ; ..etc ;
	 * node_innovations <id1> <id2> <id3> ; <id1> <id2> <id3> ; ..etc ; ]
	 * genomeend */
	Organism(ifstream& iFile) {
		char curword[20];
		functype ft;
		int node_id,node_id1,node_id2,node_id3,innov_num;
		double weight=0.0;
		Neuron *newnode=0;
		list<Neuron*>::iterator curnode;
		vector<Neuron*> nodes; // stores nodes at the first pass
		Neuron* fill_node=0; // to fill empty spaces in nodes vector
		nb_links=0;

		iFile >> curword;
		bool random_weights=(curword[0]!='0');
		id=cur_org_id++;

		bool done=false;
		//Loop until file is finished, parsing each line
		while (!done) {
			iFile >> curword;
			//Check for end of Genome
			if (strcmp(curword,"genomeend")==0) done=true;

			//Ignore comments surrounded by /* */ - they get printed to screen
			else if (strcmp(curword,"/*")==0) {
				iFile >> curword;
				while (strcmp(curword,"*/")!=0) {
					cout << curword << " ";
					iFile >> curword;
				}
				cout << endl;
			}

			// Read in an input node
			else if (strcmp(curword,"incnode")==0) {
				iFile >> curword;
				node_id=atoi(curword);
				iFile >> curword;
				ft=atoi(curword);
				//Allocate the new node
				newnode=new Neuron(ft,node_id);
				if (cur_node_id<=node_id) cur_node_id=node_id+1; // ensure node id's will be consistent
				// push its reference in a vector, sized to fit node's id
				if ((int)nodes.size()<=node_id) {
					for (int i=node_id-nodes.size()-1; i>=0; i--) nodes.push_back(fill_node);
					nodes.push_back(newnode);
				}
				else nodes[node_id]=newnode;
				//Add the node to the list of nodes
				inputs.push_back(newnode);
			}
			// Read in an hidden node
			else if (strcmp(curword,"midnode")==0) {
				iFile >> curword;
				node_id=atoi(curword);
				iFile >> curword;
				ft=atoi(curword);
				
				// size nodes to fit node's id:
				if ((int)nodes.size()<=node_id) {
					for (int i=node_id-nodes.size(); i>=0; i--) nodes.push_back(fill_node);
				}
				//Allocate the new node if needed
				if (nodes[node_id]!=0) {
					newnode=nodes[node_id];
					newnode->ftype=ft;
				}
				else newnode=new Neuron(ft,node_id);
				if (cur_node_id<=node_id) cur_node_id=node_id+1; // ensure node id's will be consistent

				iFile >> curword;
				while (strcmp(curword,";")!=0) {
					node_id=atoi(curword);
					iFile >> curword;
					innov_num=atoi(curword);
					if (random_weights) weight=randposneg()*randfloat();
					else {
						iFile >> curword;
						weight=atof(curword);
					}

					// size nodes to fit node's id:
					if ((int)nodes.size()<=node_id) {
						for (int i=node_id-nodes.size(); i>=0; i--) nodes.push_back(fill_node);
					}
					// two cases: nodes[node_id] initialized or not
					if (nodes[node_id]!=0)
						newnode->add_incoming(nodes[node_id],weight,innov_num);
					else {
						nodes[node_id]=new Neuron(node_id,ft); // ft could be wrong, but it will be set later
						newnode->add_incoming(nodes[node_id],weight,innov_num);
					}
					iFile >> curword;
				}
				//Add the node to the list of nodes
				hidden_nodes.push_back(newnode);
			}
			// Read in an output node (can't connect with another output, so this code seems safe)
			else if (strcmp(curword,"outnode")==0) {
				iFile >> curword;
				node_id=atoi(curword);
				iFile >> curword;
				ft=atoi(curword);
				
				//Allocate the new node
				newnode=new Neuron(ft,node_id);
				if (cur_node_id<=node_id) cur_node_id=node_id+1; // ensure node id's will be consistent
				iFile >> curword;
				while (strcmp(curword,";")!=0) {
					node_id=atoi(curword);
					iFile >> curword;
					innov_num=atoi(curword);
					if (random_weights) weight=randposneg()*randfloat();
					else {
						iFile >> curword;
						weight=atof(curword);
					}
					newnode->add_incoming(nodes[node_id],weight,innov_num);
					iFile >> curword;
				}
				//Add the node to the list of nodes
				outputs.push_back(newnode);
			}
			// Read link innovations, check if they exists at global
			else if (strcmp(curword,"link_innovations")==0) {
				iFile >> curword;
				node_id1=atoi(curword);
				iFile >> curword;
				node_id2=atoi(curword);
				iFile >> curword;
				innov_num=atoi(curword);
				int i;
				for (i=Innovation::global_link_innovation.size()-1; i>=0; i--) {
					if (Innovation::global_link_innovation[i]->innovation_num==innov_num) break;
				}
				if (i>=0) link_innovations.push_back(Innovation::global_link_innovation[i]);
				else {
					Link* lk=new Link(new Neuron(node_id1,nodes[node_id1]->ftype),new Neuron(node_id1,nodes[node_id1]->ftype),innov_num);
					Innovation::global_link_innovation.push_back(lk);
					link_innovations.push_back(lk);
				}
				// adjust cur_innov to be consistent:
				if ((link_innovations.back())->innovation_num>=Innovation::cur_innov) Innovation::cur_innov=(link_innovations.back())->innovation_num+1;
			}
			// Read node innovations, check if they exists at global
			else if (strcmp(curword,"node_innovations")==0) {
				iFile >> curword;
				node_id1=atoi(curword);
				iFile >> curword;
				node_id2=atoi(curword);
				iFile >> curword;
				node_id3=atoi(curword);
				int i;
				for (i=Innovation::global_node_innovation.size()-1; i>=0 && Innovation::global_node_innovation[i]!=0; i--) {
					if (Innovation::global_node_innovation[i]->midnode->id==node_id2) break;
				}
				if (Innovation::global_node_innovation[i]!=0) node_innovations.push_back(Innovation::global_node_innovation[i]);
				else {
					// we must find the link innovation num between node1 and node2:
					list<Link*>::iterator curlk;
					for (curlk=link_innovations.begin(); curlk!=link_innovations.end() 
								&& ((*curlk)->incnode->id!=node_id1 || (*curlk)->outnode->id!=node_id2); curlk++) {}
					LinkAndNode* ln=new LinkAndNode(new Neuron(node_id1,nodes[node_id1]->ftype),new Neuron(node_id1,nodes[node_id1]->ftype),
													new Neuron(node_id3,nodes[node_id3]->ftype),(*curlk)->innovation_num);
					Innovation::global_node_innovation.push_back(ln);
					node_innovations.push_back(ln);
				}
			}
		}
		fitness=0.0;
		generation=0;
		eliminate=false;
		winner=false;

		// don't forget to initialise the array of innovations
		// we do that because we want to access the innovations with instructions like:
		// global_node_innovation[node->id], but of course the initial nodes don't
		// correspond to some innovations, and the corresponding elements inside global_node_innovation
		// will never be accessed (if so, there is a problem..)
		if (Innovation::global_node_innovation.empty()) {
			LinkAndNode* fill_innov=0;
			for (int i=inputs.size()+outputs.size()-1; i>=0; i--) Innovation::global_node_innovation.push_back(fill_innov);
		}
	}

	// copy constructor
	Organism (const Organism* org) {
		// preprocessing: duplicate all the nodes, without links
		list<Neuron*>::const_iterator curnode;
		list<IncomingLink*>::const_iterator curlink;
		int sizeID=0;
		// highest ID's are typically find in the hidden layers;
		// if no hidden layer, we go through outputs neurons
		if (!org->hidden_nodes.empty()) {
			for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++)
				if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
		}
		else {
			for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++)
				if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
		}
		Neuron** tabNodes=new Neuron*[sizeID+1];

		// duplication phase
		for (curnode=org->inputs.begin(); curnode!=org->inputs.end(); curnode++)
			tabNodes[(*curnode)->id]=new Neuron(*curnode);
		for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++)
			tabNodes[(*curnode)->id]=new Neuron(*curnode);
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++)
			tabNodes[(*curnode)->id]=new Neuron(*curnode);

		// add the relations between nodes, one by one, and then add all nodes to the lists
		for (curnode=org->inputs.begin(); curnode!=org->inputs.end(); curnode++)
			inputs.push_back(tabNodes[(*curnode)->id]);
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
				tabNodes[(*curnode)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
			hidden_nodes.push_back(tabNodes[(*curnode)->id]);
		}
		for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
				tabNodes[(*curnode)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
			outputs.push_back(tabNodes[(*curnode)->id]);
		}

		//copy innovation's lists
		list<Link*>::const_iterator curlk;
		list<LinkAndNode*>::const_iterator curln;
		for (curlk=org->link_innovations.begin(); curlk!=org->link_innovations.end(); curlk++)
			link_innovations.push_back(*curlk);
		for (curln=org->node_innovations.begin(); curln!=org->node_innovations.end(); curln++)
			node_innovations.push_back(*curln);

		//copy other fields and calculate id
		id=cur_org_id++;
		nb_links=org->nb_links;
		generation=org->generation;
		fitness=org->fitness;
		eliminate=false;
		winner=false;
	}
	
	~Organism() {
		/* Erase all neurons from the lists */
		for (list<Neuron*>::iterator curnode=inputs.begin(); curnode!=inputs.end(); curnode++)
			delete *curnode;
		for (list<Neuron*>::iterator curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++)
			delete *curnode;
		for (list<Neuron*>::iterator curnode=outputs.begin(); curnode!=outputs.end(); curnode++)
			delete *curnode;
	}

	//Adds a Neuron in the id's order
	void add_neuron(Neuron* node) {
		list<Neuron*>::iterator curnode=hidden_nodes.begin();
		while (curnode!=hidden_nodes.end() && (*curnode)->id<node->id) curnode++;
		hidden_nodes.insert(curnode,node);
	}

	// reinitialise the network before a new call to load_sensors():
	void flush() {
		list<Neuron*>::iterator curnode;
		for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) (*curnode)->activation=0.0;
		for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) (*curnode)->activation=0.0;
	}

	//Takes an array of sensor values and loads it into SENSOR inputs
	void load_sensors(double* sensvals) {
		int counter=0;
		for (list<Neuron*>::iterator curnode=inputs.begin(); curnode!=inputs.end(); curnode++)
			(*curnode)->activation=sensvals[counter++];
	}

	void activate(); //Activates the net such that all outputs are active
	
	/* ******* MUTATORS ******* */
	void mutate_link_weights(); /* Change random link's weight */
	void mutate_toggle_enable(int times); /* toggle links on or off, times times */
	void mutate_gene_reenable(); /* Find first disabled link and enable it */

	/* These last kinds of mutations return false if they fail
		They can fail under certain conditions,  being unable
		to find a suitable place to make the mutation. */
	/* Mutate organism by adding a new link between 2 random neuron */
	bool mutate_add_link(int times); 
	/* Mutate organism by adding a neuron respresentation */
	bool mutate_add_node(int times);

	/* ****** MATING METHODS ***** */
	/* This method mates this Organism with another Organism g.  
		For every point in each Organism, where each Organism shares
		the innovation number, the Gene is chosen randomly from 
		either parent. If one parent has an innovation absent in 
		the other, the baby will inherit the innovation or not,
		depending of the goodness of the parent */
	Organism* mate_multipoint(const Organism* org) const;
	/* This method mates like multipoint but instead of selecting one
		or the other when the innovation numbers match, it averages their
		weights */
	Organism* mate_multipoint_avg(const Organism* org) const;
	/* This method is similar to a standard single point CROSSOVER
		operator. A point is chosen in the smaller Organism for crossing
		with the bigger one.  */
	Organism* mate_singlepoint(const Organism* org) const;

	/* ******** COMPATIBILITY CHECKING METHODS * ********/
	/* This function gives a measure of compatibility between
		two Organisms by computing a linear combination of 3
		characterizing variables of their compatibilty.
		The 3 variables represent PERCENT DISJOINT GENES, 
		PERCENT EXCESS GENES, MUTATIONAL DIFFERENCE WITHIN
		MATCHING GENES. So the formula for compatibility 
		is:  disjoint_coeff*pdg+excess_coeff*peg+mutdiff_coeff*mdmg.
		The 3 coefficients are global system parameters */
	double compatibility(const Organism* org) const;

	bool operator<(const Organism& org) const {
		return fitness > org.fitness;
	}
	
	// print to a file so that the Organism can be retrieved later
	void print_to_file(char* name);

};

#endif
