#include "Population.h" 

// to look at what is happening:
ofstream Population::trace("trace.txt");

//Create a population of size size off of Genome org
//The new Population will have the same topology as org
//with linkweights slightly perturbed
void Population::spawn(Organism* org,int size) {
	Organism* neworg;
	//Create size copies of the Genome
	//Start with perturbed linkweights
	for (int count=size-1; count>=0; count--) {
		neworg=new Organism(org);
		neworg->mutate_link_weights();
		organisms.push_back(neworg);
	}
	//Separate the new Population into species
	speciate();
}

//speciate separates the organisms into species by
//checking compatibilities against a threshold
//Any organism that is not compatible with the first
//organism in any existing species becomes a new species
void Population::speciate() {
	list<Organism*>::iterator curorg;  //For stepping through Population
	list<Species*>::iterator curspec; //Steps through species
	Species* newspecies; //For adding a new species
	
	//Step through all existing organisms
	for (curorg=organisms.begin(); curorg!=organisms.end(); curorg++) {
		//For each organism, search for a species it is compatible to
		curspec=species.begin();
		if (curspec==species.end()) {
			//Create the first species
			newspecies=new Species();
			newspecies->organisms.push_back(*curorg); //Add the current organism
			species.push_back(newspecies);
		}
		else {
			for (curspec=species.begin(); curspec!=species.end(); curspec++) {
				if ((*curorg)->compatibility(*((*curspec)->organisms.begin()))<NEAT::compat_threshold) {
					//Found compatible species, so add this organism to it
					(*curspec)->organisms.push_back(*curorg);
					break;
				}
			}
			//If we didn't find a match, create a new species
			if (curspec==species.end()) {
				newspecies=new Species();
				newspecies->organisms.push_back(*curorg); //Add the current organism
				species.push_back(newspecies);
			}
		}
	}
}

//epoch turns over a Population to the next generation based on fitness
// return false if stopped because of time
bool Population::epoch(double epoch_duration) {
	list<Species*>::iterator curspec;
	list<Species*>::iterator deadspec; //For removing empty Species
	list<Organism*>::iterator curorg,deadorg;

	#ifdef SEARCH_TIME
	// to control time:
	Timer tim;
	#endif
	
	//Use Species' ages to modify the objective fitness of organisms
	//in other words, make it more fair for younger species so they have a chance to take hold
	//Also penalize stagnant species
	//Then adjust the fitness using the species size to "share" fitness within a species.
	//Then, within each Species, mark for death those below survival_thresh*average
	for (curspec=species.begin(); curspec!=species.end(); curspec++) (*curspec)->adjust_fitness();

	//Sort the Species by max fitness
	//These need to use ORIGINAL fitness
	sortList<Species>(species);

	//Remove all empty species that survive; 
	// WARNING: organisms remain in memory till removal from main list.
	curspec=species.begin();
	#ifdef FASTEST_SEARCH
	//we keep only the 7 better "old" species to fasten the search (experimental..)
	//and keep only the 13 better babies' species
	int countSpecO=7,countSpecB=13;
	#endif
	while (curspec!=species.end()) {
		#ifdef FASTEST_SEARCH
		if ((*curspec)->organisms.empty() || (countSpecO==0 && (*curspec)->age>=1) || (countSpecB==0 && (*curspec)->age==0)) {
		#else
		if ((*curspec)->organisms.empty()) {
		#endif
		delete *curspec;
			deadspec=curspec;
			curspec++;
			species.erase(deadspec);
		}
		else {
			#ifdef FASTEST_SEARCH
			if ((*curspec)->age>=1) countSpecO--;
			else countSpecB--;
			#endif
			curspec++;
		}
	}
	
	if (species.empty()) {
		cout << "error: population extinction!" << endl;
		return false;
	}

	//Go through the organisms and add up their fitnesses to compute the overall average
	double overall_average=0.0;
	int countOrg=0;
	for (curspec=species.begin(); curspec!=species.end(); curspec++) {
		for (curorg=(*curspec)->organisms.begin(); curorg!=(*curspec)->organisms.end(); curorg++) overall_average+=(*curorg)->fitness;
		countOrg+=(*curspec)->organisms.size();
	}
	overall_average/=countOrg;
	
	//Check for Population-level stagnation
	if ((*(((*(species.begin()))->organisms).begin()))->orig_fitness>highest_fitness) {
		highest_fitness=(*(((*(species.begin()))->organisms).begin()))->orig_fitness;
		highest_last_changed=0;
		trace << "NEW POPULATION RECORD FITNESS: " << highest_fitness << endl;
	}
	else {
		highest_last_changed++;
		trace<<highest_last_changed<<" generations since last population fitness record: "<<highest_fitness<<endl;
	}

	//Check for stagnation; if there is stagnation, perform delta-coding
	if (highest_last_changed>=NEAT::dropoff_age+5) {
		trace << "PERFORMING DELTA CODING" << endl;
		highest_last_changed=0;
		int half_pop=(organisms.size()+1)/2;
		curspec=species.begin();
		(*curspec)->compute_org_offspring(0.0,half_pop);
		(*curspec)->age_of_last_improvement=(*curspec)->age;
		curspec++;
		if (curspec!=species.end()) {
			(*curspec)->compute_org_offspring(0.0,half_pop);
			(*curspec)->age_of_last_improvement=(*curspec)->age;
			curspec++;
			//Get rid of all species under the first 2
			while (curspec!=species.end()) {
				delete *curspec;
				deadspec=curspec;
				curspec++;
				species.erase(deadspec);
			}
		}
	}
	else {
		//Now compute expected number of offspring for each individual organism
		for (curspec=species.begin(); curspec!=species.end(); curspec++)
			(*curspec)->compute_org_offspring(overall_average,0);
	}

	//Perform reproduction.  Reproduction is done on a per-Species
	//basis. (So this could be paralellized potentially.)
	// we duplicate the (old) species' list to avoid iterations in new species; not needed, but cleaner
	list<Species*> speciesDup=species;
	#ifdef SEARCH_TIME
	bool is_over=false;
	#endif
	for (curspec=speciesDup.begin(); curspec!=speciesDup.end(); curspec++) {
		#ifdef SEARCH_TIME
		// if not too much species, the following call isn't too costly:
		if (tim.elapsed()>epoch_duration) {
			// time is over, so we exit this function
			// NOTE: we could also try the species reproduced, but this is same as run a new time..
			is_over=true;
			break;
		}
		#endif
		reproduce(*curspec);
	}
	#ifdef SEARCH_TIME
	if (is_over) return false;
	#endif

	//Destroy and remove the marked organisms from the main list
	curorg=organisms.begin();
	while (curorg!=organisms.end()) {
		if ((*curorg)->eliminate) {
			//Delete the organism from memory
			delete *curorg;
			//Remember where we are
			deadorg=curorg;
			curorg++;
			//Remove the organism from the master list
			organisms.erase(deadorg);
		}
		else curorg++;
	}

	//Age surviving Species and complete master Organism list
	for (curspec=species.begin(); curspec!=species.end(); curspec++) {
		//Age the species
		(*curspec)->age++;
		//Go through the organisms' babies of the curspecies and add them to the master list
		for (curorg=(*curspec)->organisms.begin(); curorg!=(*curspec)->organisms.end(); curorg++) {
			if ((*curorg)->generation==0) organisms.push_back(*curorg);
		}
	}

	return true;
}

//Perform mating and mutation to form next generation
void Population::reproduce(Species* spec) {
	list<Organism*>::iterator curorg1,curorg2;
	list<Species*>::iterator curspec;
	Organism *baby_model,*baby; // a baby of 2 organisms
	Organism* mate; // a partner for the current organism
	int shift,counter=0,num_parents=spec->organisms.size();
	double compat,best_compat; // for testing compatibility
	double randnum;

	list<Organism*> specOrgDup=spec->organisms;
	// iterate through the designated parents, and for each of them find a partner
	for (curorg1=specOrgDup.begin(); curorg1!=specOrgDup.end(); curorg1++) {
		if ((*curorg1)->generation==0) break; // skip newly created organisms from previous iterations
		// we search for mates on a pseudo-gaussian centered at the position of curorg1;
		// this allow for mating with similar individuals (if large species..)
		if (num_parents>1) {
			shift=0;
			do shift=minm(max(-counter,(int)floor(p_gaussrand())),specOrgDup.size()-1-counter);
			while (shift==0);
			counter++;
			curorg2=curorg1;
			if (shift>0) {
				while (shift-->0) curorg2++;
			}
			else {
				while (shift++<0) curorg2--;
			}
			mate=(*curorg2);
		}
		else {
			// we have to find the partner in another species: this is done at random
			// if this is the last individual (shouldn't arrive if good parameters..), we clone it, mutate and mate
			if (organisms.size()==1) {
				mate=new Organism(*curorg1);
				mate->mutate_link_weights();
				mate->mutate_toggle_enable((int)floor(NEAT::frac_toggle_link*(mate->nb_links)));
				organisms.push_back(mate);
				spec->organisms.push_back(mate);
			}
			else {
				// we look for the organism with a structure as close as possible
				// means the old species will disappear, fusionning with an existing one
				// NOTE: this is costly in computation, could be replaced by simple mutate/clone/mate if too slow
				best_compat=MAX_DOUBLE;
				for (curorg2=organisms.begin(); curorg2!=organisms.end(); curorg2++) {
					if (curorg2!=curorg1) {
						compat=((*curorg1)->compatibility(*curorg2));
						if (compat<best_compat) {
							best_compat=compat;
							mate=(*curorg2);
						}
					}
				}
			}
		}

		// now, the last step: we can create the babies (as much as the organism must have)
		//Perform mating based on probabilities of different mating types
		randnum=randfloat();
		if (randnum<NEAT::mate_multipoint_prob)
			baby_model=(*curorg1)->mate_multipoint(mate);
		else if (randnum<(NEAT::mate_multipoint_prob+NEAT::mate_multipoint_avg_prob)
							/(NEAT::mate_multipoint_prob+NEAT::mate_multipoint_avg_prob+NEAT::mate_singlepoint_prob))
			baby_model=(*curorg1)->mate_multipoint_avg(mate);
		else baby_model=(*curorg1)->mate_singlepoint(mate);

		// we obtain the babies by mutation over the "model" just created
		for (int i=(*curorg1)->expected_offspring-1; i>=0; i--) {
			baby=new Organism(baby_model);
			//Do the mutation of baby depending on probabilities of various mutations
			bool mut_struct=false;
			randnum=randfloat();
			if (randnum<NEAT::mutate_add_node_prob) {
				if (baby->mutate_add_node((int)floor(NEAT::frac_mut_node*(baby->inputs.size()+baby->hidden_nodes.size()+baby->outputs.size()))))
					mut_struct=true;
			}
			if (randnum<NEAT::mutate_add_link_prob) {
				if (baby->mutate_add_link((int)floor(NEAT::frac_mut_link*(baby->nb_links)))) 
					mut_struct=true;
			}
			if (!mut_struct) {
				//Only do other mutations when not doing stuctural mutations
				// NOTE: maybe we could combine both ?
				randnum=randfloat();
				if (randnum<NEAT::mutate_link_weights_prob)
					baby->mutate_link_weights();
				if (randnum<NEAT::mutate_toggle_enable_prob)
					baby->mutate_toggle_enable((int)floor(NEAT::frac_toggle_link*(baby->nb_links)));
				if (randnum<NEAT::mutate_gene_reenable_prob)
					baby->mutate_gene_reenable();
			}

			//Add the baby to its proper Species
			for (curspec=species.begin(); curspec!=species.end(); curspec++) {
				if (baby->compatibility(*((*curspec)->organisms.begin()))<NEAT::compat_threshold) {
					//Found compatible species, so add this organism to it
					(*curspec)->organisms.push_back(baby);
					break;
				}
			}
			//If we didn't find a match, create a new species
			if (curspec==species.end()) {
				Species* newspecies=new Species();
				newspecies->organisms.push_back(baby);
				species.push_back(newspecies);
			}
		}
		delete baby_model;
	}
	specOrgDup.clear();
}

//Print Population to a file specified by a string
void Population::print_to_file(char* filename){
	list<Organism*>::iterator curorg;
	list<Species*>::iterator curspec;
	ofstream outFile(filename);
	//Make sure it worked
	if (!outFile.is_open()) {
		cerr << "Can't open " << filename << " for output" << endl;
		return;
	}
	int count_babies;
	//Print all the Organisms of each species to the outFile
	for (curspec=species.begin(); curspec!=species.end(); curspec++) {
		outFile << "Species number " << (*curspec)->id << ",age : " << (*curspec)->age 
		<< ",containing " << (*curspec)->organisms.size() << " organisms" << endl;
		count_babies=0;
		for (curorg=(*curspec)->organisms.begin(); curorg!=(*curspec)->organisms.end(); curorg++) {
			if ((*curorg)->generation>0) 
				outFile << "org " << (*curorg)->id << " with " << (*curorg)->hidden_nodes.size() 
				<< " hidden nodes, and fitness " << (*curorg)->orig_fitness << endl;
			else count_babies++;
		}
		outFile << "(" << count_babies << " babies)" << endl;
		outFile << endl;
	}
	outFile.close();
}
