#include "GeneticSpecies.h"

/* --------------------------------------------- */
/* SPECIES RELATED METHODS                    */
/* --------------------------------------------- */

bool Species::add_Organism(Organism *o){
	organisms.push_back(o);
	return true;
}

Organism *Species::first() {
	return *(organisms.begin());
}

//Can change the fitness of the organisms in the Species to be higher 
//for very new species (to protect them).
//Divides the fitness by the size of the Species, so that fitness is
//"shared" by the species
void Species::adjust_fitness() {
	list<Organism*>::iterator curorg;
	int num_parents;
	int count;
	int age_debt; 
	//cout<<"Species "<<id<<" last improved "<<(age-age_of_last_improvement)<<" steps ago when it moved up to "<<max_fitness_ever<<endl;
	age_debt=(age-age_of_last_improvement+1)-dropoff_age;
	if (age_debt==0) age_debt=1;
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		//Remember the original fitness before it gets modified
		(*curorg)->orig_fitness=(*curorg)->fitness;
		//Make fitness decrease after a stagnation point dropoff_age
		//Added an if to keep species pristine until the dropoff point
		if (age_debt>=1) {
			//Possible graded dropoff
			//((*curorg)->fitness)=((*curorg)->fitness)*(-atan(age_debt));
			//Extreme penalty for a long period of stagnation (divide fitness by 100)
			((*curorg)->fitness)=((*curorg)->fitness)*0.01;
			cout<<"dropped fitness to "<<((*curorg)->fitness)<<endl;
		}
		//Give a fitness boost up to some young age (niching)
		//The age_significance parameter is a system parameter
		//  if it is 1, then young species get no fitness boost
		if (age<=10) ((*curorg)->fitness)=((*curorg)->fitness)*age_significance; 
		//Do not allow negative fitness
		if (((*curorg)->fitness)<0.0) (*curorg)->fitness=0.0001; 
		//Share fitness with the species
		(*curorg)->fitness=((*curorg)->fitness)/(organisms.size());
	}

	//Sort the population and mark for death those after survival_thresh*pop_size
	organisms.sort();

	//Update age_of_last_improvement here
	if (((*(organisms.begin()))->orig_fitness)>max_fitness_ever) {
		age_of_last_improvement=age;
		max_fitness_ever=((*(organisms.begin()))->orig_fitness);
	}
	//Decide how many get to reproduce based on survival_thresh*pop_size
	//Adding 1.0 ensures that at least one will survive
	num_parents=(int) floor((survival_thresh*((double) organisms.size()))+1.0);
	//Mark for death those who are ranked too low to be parents
	curorg=organisms.begin();
	(*curorg)->champion=true;  //Mark the champ as such
	for(count=1;count<=num_parents;count++) {
		if (curorg!=organisms.end())
			++curorg;
	}
	while(curorg!=organisms.end()) {
		(*curorg)->eliminate=true;  //Mark for elimination
		++curorg;
	}             
}

double Species::compute_average_fitness() {
	list<Organism*>::iterator curorg;
	double total=0.0;
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		total+=(*curorg)->fitness;
	}
	ave_fitness=total/(organisms.size());
	return ave_fitness;
}

double Species::compute_max_fitness() {
	double max=0.0;
	list<Organism*>::iterator curorg;
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		if (((*curorg)->fitness)>max)
			max=(*curorg)->fitness;
	}
	max_fitness=max;
	return max;
}

//Compute the collective offspring the entire
//species (the sum of all organism's offspring)
//is assigned
//skim is fractional offspring left over from a
//previous species that was counted.
//These fractional parts are kept unil they add
//up to 1
double Species::count_offspring(double skim) {
	list<Organism*>::iterator curorg;
	int e_o_intpart;  //The floor of an organism's expected offspring
	double e_o_fracpart; //Expected offspring fractional part
	double skim_intpart;  //The whole offspring in the skim
	expected_offspring=0;
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		e_o_intpart=(int) floor((*curorg)->expected_offspring);
		e_o_fracpart=fmod((*curorg)->expected_offspring,1.0);
		expected_offspring+=e_o_intpart;
		//Skim off the fractional offspring
		skim+=e_o_fracpart;
		//NOTE:  Some precision is lost by computer
		//       Must be remedied later
		if (skim>1.0) {
			skim_intpart=floor(skim);
			expected_offspring+=(int) skim_intpart;
			skim-=skim_intpart;
		}
	}
	return skim;
}

//Remove an organism from Species
bool Species::remove_org(Organism *org) {
	list<Organism*>::iterator curorg;
	curorg=organisms.begin();
	while((curorg!=organisms.end())&&((*curorg)!=org)) ++curorg;
	if (curorg==organisms.end()) return false;
	else {
		organisms.erase(curorg);
		return true;
	}
}

Organism *Species::get_champ() {
	double champ_fitness=-1.0;
	Organism *thechamp;
	list<Organism*>::iterator curorg;
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		if (((*curorg)->fitness)>champ_fitness) {
			thechamp=(*curorg);
			champ_fitness=thechamp->fitness;
		}
	}
	return thechamp;
}

//Print Species to a file outFile
bool Species::print_to_file(ofstream &outFile) {
	list<Organism*>::iterator curorg;
	//Print a comment on the Species info
	outFile<<endl<<"/* Species #"<<id<<" : (Size "<<organisms.size()<<") (AF "<<ave_fitness<<") (Age "<<age<<")  */"<<endl<<endl;
	//Show user what's going on
	cout<<endl<<"/* Species #"<<id<<" : (Size "<<organisms.size()<<") (AF "<<ave_fitness<<") (Age "<<age<<")  */"<<endl;
	//Print all the Organisms' Genomes to the outFile
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		//Put the fitness for each organism in a comment
		outFile<<endl<<"/* Organism #"<<((*curorg)->gnome)->genome_id<<" Fitness: "<<(*curorg)->fitness<<" Error: "<<(*curorg)->error<<" */"<<endl;
		//If it is a winner, mark it in a comment
		if ((*curorg)->winner) outFile<<"/* ##------$ WINNER "<<((*curorg)->gnome)->genome_id<<" SPECIES #"<<id<<" $------## */"<<endl;
		((*curorg)->gnome)->print_to_file(outFile);
		//We can confirm by writing the genome #'s to the screen
		//cout<<((*curorg)->gnome)->genome_id<<endl;
	}
	return true;
}

//This is used for list sorting of Species by fitness of best organism
//highest fitness first
bool Species::operator<(Species *x) {
	//cout<<"Comparing "<<((*((x->organisms).begin()))->orig_fitness)<<" and "<<((*((y->organisms).begin()))->orig_fitness)<<": "<<(((*((x->organisms).begin()))->orig_fitness) > ((*((y->organisms).begin()))->orig_fitness))<<endl;
	return (*(organisms.begin()) < *((x->organisms).begin()));
}
