#include "Fighter.h"

const int MAX_LOOPS=300; // max logic moves without hurt (total 15 seconds)
const double MAX_LIFE=50.0; // maximum life
const double LEARNING_RATE=0.6; // parameter for network learning

// ghost keys for testing:
Uint8* emptyKeys;

// "Organism" representing human:
Organism* human;

// vector of human actions if this mode is on for evolving networks:
vector<Action> humanActions;

// must be called before any fight
void Fighter::init_fight() {
	emptyKeys=0;
	human=0;
	humanActions.clear();
	init_inputs();
}

// function to add an incoming link without redondancy to a list: we compare the adresses of objects
void Fighter::insertWR(IncomingLink* link,list<IncomingLink*>& incList) {
	list<IncomingLink*>::iterator curlink;
	for (curlink=incList.begin(); curlink!=incList.end(); curlink++) {
		if ((*curlink)==link) break;
	}
	if (curlink==incList.end()) {
		incList.push_back(link);
	}
}

// fighter learning: increase or decrease connections between active inputs and outputs
void Fighter::learnWeights(Organism* org,Control ctrl,list<list<Neuron*> >& layers,bool increase) {
	list<list<Neuron*> >::iterator layers_iter;
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;

	// here is the vector of nodes to explore while learning:
	vector<Neuron*> nodesToExplore;
	// and the vector of incoming links to explore (to keep the track):
	vector<IncomingLink*> linksToExplore;
	// finally, list of incoming links which will be effectively modified:
	list<IncomingLink*> linksToAlter;

	// changes link's weights along "branches" starting from active outputs to active inputs
	// a further possibility: take into account the topology..
	curnode=(*(layers.begin())).begin();
	for (int i=ctrl-1; i>=0; i--) curnode++;
	nodesToExplore.push_back(*curnode);

	// first mark all nodes as not visited:
	for (curnode=org->inputs.begin(); curnode!=org->inputs.end(); curnode++) (*curnode)->visited=false;
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) (*curnode)->visited=false;
	for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) (*curnode)->visited=false;

	// run through the "tree" (we don't take loops into account, so corresponding weights are not modified)
	Neuron* node;
	IncomingLink* link=0;
	while (!nodesToExplore.empty()) {
		node=nodesToExplore.back();
		nodesToExplore.pop_back();
		if (!linksToExplore.empty()) { // this condition is always true except at the beginning
			link=linksToExplore.back();
			linksToExplore.pop_back();
		}

		// add each new node and link from the previous layer (if inputs are the starting point):
		for (curlink=node->incoming.begin(); curlink!=node->incoming.end(); curlink++) {
			if ((*curlink)->node->depth<node->depth && !(*curlink)->node->visited) {
				nodesToExplore.push_back((*curlink)->node);
				linksToExplore.push_back(*curlink);
			}
		}

		// if depth is zero and input is active, add links of the path to linksToAlter, including link
		if (node->depth==0 && node->activation!=0.0) {
			insertWR(link,linksToAlter);
			int i=linksToExplore.size()-1;
			while (i>=0 && linksToExplore[i]->node->depth==0) i--;
			if (i>=0) {
				for (; i>=0; i--) insertWR(linksToExplore[i],linksToAlter);
			}
		}

		// mark current node as visited:
		node->visited=true;
	}
	// change links' weights:
	if (increase) {
		for (curlink=linksToAlter.begin(); curlink!=linksToAlter.end(); curlink++)
			(*curlink)->weight += (LEARNING_RATE*fabs((*curlink)->weight));//+MIN_LEARNING);
	}
	else {
		for (curlink=linksToAlter.begin(); curlink!=linksToAlter.end(); curlink++)
			(*curlink)->weight -= (LEARNING_RATE*fabs((*curlink)->weight));//+MIN_LEARNING);
	}
}

// fighter's evaluation: can just be relative, so we run a fight against best_fit;
// no learning during evaluation, to get a better diversity
void Fighter::launch_test_fight(Organism* org,Organism* best_fit_org) {
	if (humanActions.empty()) {
		// basic first case for the first organism: fitness=MAX_LIFE
		if (best_fit_org==0) {
			org->fitness=MAX_LIFE;
			return;
		}
		best_fit_org->flush();
	}
	// first, clear the network from previous uses:
	org->flush();

	// positions of each player:
	SDL_Rect position1,position2;
	position1.x=200;
	position1.y=380;
	position2.x=600;
	position2.y=380;

	// energies of each player, 1 is the challenger versus best_fit or human player:
	double oldLife1,life1=MAX_LIFE,oldLife2,life2=MAX_LIFE;
	int j=0,diffX=400,lastAction1=-1,lastAction2=-1;
	Control p1_ctrl1=0,p1_ctrl2=0,p2_ctrl1=0,p2_ctrl2=0;
	ActionsAndPositions actsAndPos;

	while (life1>0.0 && life2>0.0 && j++ <MAX_LOOPS) {
		// do some actions...
		oldLife1=life1;
		oldLife2=life2;

		if (lastAction1>=0) {
			if (position1.x+distX[17*(position1.y<GROUND ? 2 : (position1.y==GROUND ? 1 : 0))+lastAction2]
				<= position2.x+position2.w-distX[17*(position2.y<GROUND ? 2 : (position2.y==GROUND ? 1 : 0))+lastAction1])
				diffX=position2.x+position2.w-distX[17*(position2.y<GROUND ? 2 : (position2.y==GROUND ? 1 : 0))+lastAction1]
						-position1.x+distX[17*(position1.y<GROUND ? 2 : (position1.y==GROUND ? 1 : 0))+lastAction2];
			else
				diffX=position1.x+position1.w-distX[17*(position1.y<GROUND ? 2 : (position1.y==GROUND ? 1 : 0))+lastAction2]
						-position2.x+distX[17*(position2.y<GROUND ? 2 : (position2.y==GROUND ? 1 : 0))+lastAction1];
		}

		if (humanActions.empty()) {
			get_controls(emptyKeys,org,best_fit_org,diffX,position2.y-position1.y,p1_ctrl1,p1_ctrl2,p2_ctrl1,p2_ctrl2);
			actsAndPos=inputs_handler(org,best_fit_org,life1,position1,life2,position2);
		}
		else {
			get_controls(emptyKeys,org,human,diffX,position2.y-position1.y,p1_ctrl1,p1_ctrl2,p2_ctrl1,p2_ctrl2);
			actsAndPos=inputs_handler(org,human,life1,position1,life2,position2,FALSE,FALSE,humanActions);
		}

		// update actions:
		lastAction1=actsAndPos.act1;
		lastAction2=actsAndPos.act2;

		// reset counter if life decrease:
		if (life1!=oldLife1 || life2!=oldLife2) j=0;
	}

	org->fitness=life1+MAX_LIFE-life2;
	if (humanActions.empty()) best_fit_org->fitness=life2+MAX_LIFE-life1; // the best_fit organism is always challenged
	else org->winner=(org->fitness>=2*MAX_LIFE-0.01);
}

bool Fighter::should_exit(SDL_Event *ev) {
	switch(ev->type) {
		case SDL_MOUSEBUTTONUP:
			break;
		case SDL_KEYUP:
			switch(ev->key.keysym.sym) {
				case SDLK_ESCAPE:
					return true;
				default:
					break;
			}
			break;
		case SDL_QUIT:
			return true;
	}
	return false;
}

// function to (re)draw the lifebar
void Fighter::printLifeBar(SDL_Surface* screen,double life1,double life2) {
	SDL_Rect rectLife;
	rectLife.y=32;
	rectLife.h=25;
	rectLife.x=66;
	rectLife.w=(int)(444*(MAX_LIFE-life1)/MAX_LIFE);
	fill_square(screen,&rectLife,red);
	rectLife.x=(int)(974-444*((MAX_LIFE-life2)/MAX_LIFE));
	rectLife.w=(int)(444*(MAX_LIFE-life2)/MAX_LIFE);
	fill_square(screen,&rectLife,red);
}

// main function to run a fight against human
int Fighter::fight_main(int argc,char** argv,bool memHumanForEval) {
	SDL_Surface *screen;
	int i;
	int bpp = 0;
	int flags = SDL_DOUBLEBUF | SDL_HWSURFACE | SDL_FULLSCREEN;

	SDL_Init(SDL_INIT_VIDEO);
	atexit(SDL_Quit);

	Organism *neural_fighter,*challenger=0;
	if (!memHumanForEval) {
		std::ifstream fileOrg(argv[2]);
		if (!fileOrg) {
			cerr << "error: can't read network file" << endl;
			return -1;
		}
		neural_fighter=new Organism(fileOrg);
		fileOrg.close();
		if (argc>=4) {
			std::ifstream fileOrg(argv[3]);
			if (!fileOrg) {
				cerr << "error: can't read network file" << endl;
				return -1;
			}
			challenger=new Organism(fileOrg);
			fileOrg.close();
		}
	}
	else {
		std::ifstream fileOrg("orgs/fight/firstFighter.txt");
		if (!fileOrg) {
			cerr << "error: can't find first fighter" << endl;
			return -1;
		}
		neural_fighter=new Organism(fileOrg);
		fileOrg.close();
	}

	for(i = 1; i < argc; ++i) {
		if(strncmp(argv[i], "-s", 2) == 0)
			flags &= ~SDL_DOUBLEBUF;
		else if(strncmp(argv[i], "-f", 2) == 0)
			flags |= SDL_FULLSCREEN;
		else
			bpp = atoi(&argv[i][1]);
	}

	screen = SDL_SetVideoMode(FightInputs::SCREEN_W,FightInputs::SCREEN_H,bpp,flags);
	if(!screen) {
		fprintf(stderr, "Failed to open screen!\n");
		return 1;
	}

	SDL_WM_SetCaption("Fixed Rate Fight", "Fight");
	SDL_ShowCursor(0);

	SDL_Rect upLeft;
	upLeft.x=0;
	upLeft.y=0;
	SDL_Surface *bg=IMG_Load("picsFight/background.png");
	
	// load all game pictures:
	SDL_Rect position1,position2; // respective positions of rectangles describing each player
	position1.x=200;
	position1.y=380;
	position2.x=600;
	position2.y=380;

	// actions for drawing: left,
	SDL_Surface* waitingL=IMG_Load("picsFight/left/waitingL.png");
	SDL_Surface* guardL=IMG_Load("picsFight/left/guardL.png");
	SDL_Surface* downL=IMG_Load("picsFight/left/downL.png");
	SDL_Surface* downGuardL=IMG_Load("picsFight/left/downGuardL.png");
	SDL_Surface* kneeL=IMG_Load("picsFight/left/kneeL.png");
	SDL_Surface* punch1L=IMG_Load("picsFight/left/punch1L.png");
	SDL_Surface* punch2L=IMG_Load("picsFight/left/punch2L.png");
	SDL_Surface* downPunch1L=IMG_Load("picsFight/left/downPunch1L.png");
	SDL_Surface* downPunch2L=IMG_Load("picsFight/left/downPunch2L.png");
	SDL_Surface* airPunch1L=IMG_Load("picsFight/left/airPunch1L.png");
	SDL_Surface* airPunch2L=IMG_Load("picsFight/left/airPunch2L.png");
	SDL_Surface* kick1L=IMG_Load("picsFight/left/kick1L.png");
	SDL_Surface* kick2L=IMG_Load("picsFight/left/kick2L.png");
	SDL_Surface* downKick1L=IMG_Load("picsFight/left/downKick1L.png");
	SDL_Surface* downKick2L=IMG_Load("picsFight/left/downKick2L.png");
	SDL_Surface* airKickL=IMG_Load("picsFight/left/airKickL.png");
	SDL_Surface* reverseKick1L=IMG_Load("picsFight/left/reverseKick1L.png");
	SDL_Surface* reverseKick2L=IMG_Load("picsFight/left/reverseKick2L.png");
	SDL_Surface* jumpL=IMG_Load("picsFight/left/jumpL.png");

	// and right:
	SDL_Surface* waitingR=IMG_Load("picsFight/right/waitingR.png");
	SDL_Surface* guardR=IMG_Load("picsFight/right/guardR.png");
	SDL_Surface* downR=IMG_Load("picsFight/right/downR.png");
	SDL_Surface* downGuardR=IMG_Load("picsFight/right/downGuardR.png");
	SDL_Surface* kneeR=IMG_Load("picsFight/right/kneeR.png");
	SDL_Surface* punch1R=IMG_Load("picsFight/right/punch1R.png");
	SDL_Surface* punch2R=IMG_Load("picsFight/right/punch2R.png");
	SDL_Surface* downPunch1R=IMG_Load("picsFight/right/downPunch1R.png");
	SDL_Surface* downPunch2R=IMG_Load("picsFight/right/downPunch2R.png");
	SDL_Surface* airPunch1R=IMG_Load("picsFight/right/airPunch1R.png");
	SDL_Surface* airPunch2R=IMG_Load("picsFight/right/airPunch2R.png");
	SDL_Surface* kick1R=IMG_Load("picsFight/right/kick1R.png");
	SDL_Surface* kick2R=IMG_Load("picsFight/right/kick2R.png");
	SDL_Surface* downKick1R=IMG_Load("picsFight/right/downKick1R.png");
	SDL_Surface* downKick2R=IMG_Load("picsFight/right/downKick2R.png");
	SDL_Surface* airKickR=IMG_Load("picsFight/right/airKickR.png");
	SDL_Surface* reverseKick1R=IMG_Load("picsFight/right/reverseKick1R.png");
	SDL_Surface* reverseKick2R=IMG_Load("picsFight/right/reverseKick2R.png");
	SDL_Surface* jumpR=IMG_Load("picsFight/right/jumpR.png");

	// fill corresponding tabs:
	SDL_Surface* tabPicActsL_down[17]=
		{downL,downL,downL,downL,downL,downL,downL,downL,downL,
		downGuardL,downPunch1L,downPunch2L,downKick1L,downKick2L,reverseKick1L,
			reverseKick2L,kneeL};
	SDL_Surface* tabPicActsL[17]=
		{waitingL,waitingL,waitingL,waitingL,waitingL,waitingL,downL,downL,downL,
		guardL,punch1L,punch2L,kick1L,kick2L,reverseKick1L,reverseKick2L,kneeL};
	SDL_Surface* tabPicActsL_air[17]=
		{jumpL,jumpL,jumpL,jumpL,jumpL,jumpL,downL,downL,downL,
		jumpL,airPunch1L,airPunch2L,airKickL,airKickL,airKickL,airKickL,kneeL};

	SDL_Surface* tabPicActsR_down[17]=
		{downR,downR,downR,downR,downR,downR,downR,downR,downR,
		downGuardR,downPunch1R,downPunch2R,downKick1R,downKick2R,reverseKick1R,
			reverseKick2R,kneeR};
	SDL_Surface* tabPicActsR[17]=
		{waitingR,waitingR,waitingR,waitingR,waitingR,waitingR,downR,downR,downR,
		guardR,punch1R,punch2R,kick1R,kick2R,reverseKick1R,reverseKick2R,kneeR};
	SDL_Surface* tabPicActsR_air[17]=
		{jumpR,jumpR,jumpR,jumpR,jumpR,jumpR,downR,downR,downR,
		jumpR,airPunch1R,airPunch2R,airKickR,airKickR,airKickR,airKickR,kneeR};

	// reorganize neural networks to facilitate learning:
	list<list<Neuron*> > nFighter_layers,chall_layers;
	if (neural_fighter->layers.empty()) {
		neural_fighter->arrange_topology(nFighter_layers);
		neural_fighter->layers=nFighter_layers;
	}
	else nFighter_layers=neural_fighter->layers; // avoid repetition of calcul
	if (challenger!=0) {
		if (challenger->layers.empty()) {
			challenger->arrange_topology(chall_layers);
			challenger->layers=chall_layers;
		}
		else chall_layers=challenger->layers;
	}

	Uint8* keys = SDL_GetKeyState(&i);
	bool downPressed1,downPressed2; // booleans telling if the DOWN key is pressed
	double oldLife1,life1=MAX_LIFE,oldLife2,life2=MAX_LIFE; // energies of each player, 1 is the human player
	ActionsAndPositions actsAndPos;
	Control p1_ctrl1=0,p1_ctrl2=0,p2_ctrl1=0,p2_ctrl2=0;
	int j=0,hShift,vShift,baseShift1,baseShift2,diffX=400,lastAction1=-1,lastAction2=-1;
	while (life1>0.0 && life2>0.0 && j++ <MAX_LOOPS) {
		SDL_Event ev;
		// Handle input
		while(SDL_PollEvent(&ev) > 0)
			if (should_exit(&ev)) return 0;

		// do some actions...
		downPressed1=false;
		downPressed2=false;
		oldLife1=life1;
		oldLife2=life2;

		if (lastAction1>=0) {
			if (position1.x+distX[17*(position1.y<GROUND ? 2 : (position1.y==GROUND ? 1 : 0))+lastAction2]
				<= position2.x+position2.w-distX[17*(position2.y<GROUND ? 2 : (position2.y==GROUND ? 1 : 0))+lastAction1])
				diffX=position2.x+position2.w-distX[17*(position2.y<GROUND ? 2 : (position2.y==GROUND ? 1 : 0))+lastAction1]
						-position1.x+distX[17*(position1.y<GROUND ? 2 : (position1.y==GROUND ? 1 : 0))+lastAction2];
			else
				diffX=position1.x+position1.w-distX[17*(position1.y<GROUND ? 2 : (position1.y==GROUND ? 1 : 0))+lastAction2]
						-position2.x+distX[17*(position2.y<GROUND ? 2 : (position2.y==GROUND ? 1 : 0))+lastAction1];
		}

		if (argc>=4) {
			get_controls(emptyKeys,challenger,neural_fighter,diffX,position2.y-position1.y,p1_ctrl1,p1_ctrl2,p2_ctrl1,p2_ctrl2);
			actsAndPos=inputs_handler(challenger,neural_fighter,life1,position1,life2,position2,downPressed1,downPressed2);
		}
		else {
			get_controls(keys,human,neural_fighter,diffX,position2.y-position1.y,p1_ctrl1,p1_ctrl2,p2_ctrl1,p2_ctrl2);
			actsAndPos=inputs_handler(human,neural_fighter,life1,position1,life2,position2,downPressed1,downPressed2);
		}
		// stores human action if needed:
		if (memHumanForEval) humanActions.push_back(actsAndPos.act1);

		// learning for virtual organisms:
		if (life1<oldLife1 && oldLife1-life1>oldLife2-life2) {
			if (p2_ctrl1>=0) learnWeights(neural_fighter,p2_ctrl1,nFighter_layers,true);
			if (p2_ctrl1>=0) learnWeights(neural_fighter,p2_ctrl2,nFighter_layers,true);
			if (challenger!=0) {
				if (p1_ctrl1>=0) learnWeights(challenger,p1_ctrl1,chall_layers,false);
				if (p1_ctrl2>=0) learnWeights(challenger,p1_ctrl2,chall_layers,false);
			}
		}
		else if (life2<oldLife2 && oldLife2-life2>oldLife1-life1) {
			if (p2_ctrl1>=0) learnWeights(neural_fighter,p2_ctrl1,nFighter_layers,false);
			if (p2_ctrl1>=0) learnWeights(neural_fighter,p2_ctrl2,nFighter_layers,false);
			if (challenger!=0) {
				if (p1_ctrl1>=0) learnWeights(challenger,p1_ctrl1,chall_layers,true);
				if (p1_ctrl2>=0) learnWeights(challenger,p1_ctrl2,chall_layers,true);
			}
		}

		// Update sprites:
		SDL_BlitSurface(bg,NULL,screen,&upLeft);
		
		// print status bar and update counter
		if (life1!=oldLife1 || life2!=oldLife2) j=0;
		printLifeBar(screen,life1,life2);

		if (GROUND-position1.y>0) baseShift1=2;
		else if (GROUND-position1.y==0) baseShift1=1;
		else baseShift1=0;
		if (GROUND-position2.y>0) baseShift2=2;
		else if (GROUND-position2.y==0) baseShift2=1;
		else baseShift2=0;
		if (position1.x+distX[17*baseShift1+actsAndPos.act1]
			<= position2.x+position2.w-distX[17*baseShift2+actsAndPos.act2]+5) {
			// player 1:
			if (position1.y>=GROUND) {
				if (downPressed1) {
					position1.y=GROUND+STEP_DOWN+tabHeightActs_down[0]-tabHeightActs_down[actsAndPos.act1];
					SDL_BlitSurface(tabPicActsL_down[actsAndPos.act1],NULL,screen,&position1);
					position1.y=GROUND+STEP_DOWN;
				}
				else {
					position1.y=GROUND+tabHeightActs[0]-tabHeightActs[actsAndPos.act1];
					SDL_BlitSurface(tabPicActsL[actsAndPos.act1],NULL,screen,&position1);
					position1.y=GROUND;
				}
			}
			else {
				vShift=0;
				if (!actsAndPos.lvC1) vShift=tabHeightActs_air[actsAndPos.act1]-actsAndPos.r1.h;
				position1.y-=vShift;
				SDL_BlitSurface(tabPicActsL_air[actsAndPos.act1],NULL,screen,&position1);
				position1.y+=vShift;
			}
			// player 2:
			if (position2.y>=GROUND) {
				if (downPressed2) {
					hShift=maxm(tabWidthActs_down[actsAndPos.act2]-actsAndPos.r2.w,0);
					position2.x-=hShift;
					position2.y=GROUND+STEP_DOWN+tabHeightActs_down[0]-tabHeightActs_down[actsAndPos.act2];
					SDL_BlitSurface(tabPicActsR_down[actsAndPos.act2],NULL,screen,&position2);
					position2.y=GROUND+STEP_DOWN;
				}
				else {
					hShift=maxm(tabWidthActs[actsAndPos.act2]-actsAndPos.r2.w,0);
					position2.x-=hShift;
					position2.y=GROUND+tabHeightActs[0]-tabHeightActs[actsAndPos.act2];
					SDL_BlitSurface(tabPicActsR[actsAndPos.act2],NULL,screen,&position2);
					position2.y=GROUND;
				}
			}
			else {
				hShift=maxm(tabWidthActs_air[actsAndPos.act2]-actsAndPos.r2.w,0);
				vShift=0;
				if (!actsAndPos.lvC2) vShift=tabHeightActs_air[actsAndPos.act2]-actsAndPos.r2.h;
				position2.x-=hShift;
				position2.y-=vShift;
				SDL_BlitSurface(tabPicActsR_air[actsAndPos.act2],NULL,screen,&position2);
				position2.y+=vShift;
			}
			position2.x+=hShift;
		}
		else {
			// player 1:
			if (position1.y>=GROUND) {
				if (downPressed1) {
					hShift=maxm(tabWidthActs_down[actsAndPos.act1]-actsAndPos.r1.w,0);
					position1.x-=hShift;
					position1.y=GROUND+STEP_DOWN+tabHeightActs_down[0]-tabHeightActs_down[actsAndPos.act1];
					SDL_BlitSurface(tabPicActsR_down[actsAndPos.act1],NULL,screen,&position1);
					position1.y=GROUND+STEP_DOWN;
				}
				else {
					hShift=maxm(tabWidthActs[actsAndPos.act1]-actsAndPos.r1.w,0);
					position1.x-=hShift;
					position1.y=GROUND+tabHeightActs[0]-tabHeightActs[actsAndPos.act1];
					SDL_BlitSurface(tabPicActsR[actsAndPos.act1],NULL,screen,&position1);
					position1.y=GROUND;
				}
			}
			else {
				hShift=maxm(tabWidthActs_air[actsAndPos.act1]-actsAndPos.r1.w,0);
				vShift=0;
				if (!actsAndPos.lvC1) vShift=tabHeightActs_air[actsAndPos.act1]-actsAndPos.r1.h;
				position1.x-=hShift;
				position1.y-=vShift;
				SDL_BlitSurface(tabPicActsR_air[actsAndPos.act1],NULL,screen,&position1);
				position1.y+=vShift;
			}
			position1.x+=hShift;
			// player 2:
			if (position2.y>=GROUND) {
				if (downPressed2) {
					position2.y=GROUND+STEP_DOWN+tabHeightActs_down[0]-tabHeightActs_down[actsAndPos.act2];
					SDL_BlitSurface(tabPicActsL_down[actsAndPos.act2],NULL,screen,&position2);
					position2.y=GROUND+STEP_DOWN;
				}
				else {
					position2.y=GROUND+tabHeightActs[0]-tabHeightActs[actsAndPos.act2];
					SDL_BlitSurface(tabPicActsL[actsAndPos.act2],NULL,screen,&position2);
					position2.y=GROUND;
				}
			}
			else {
				vShift=0;
				if (!actsAndPos.lvC2) vShift=tabHeightActs_air[actsAndPos.act2]-actsAndPos.r2.h;
				position2.y-=vShift;
				SDL_BlitSurface(tabPicActsL_air[actsAndPos.act2],NULL,screen,&position2);
				position2.y+=vShift;
			}
		}

		// Make the new frame visible
		SDL_Flip(screen);

		// make things visible for human
		SDL_Delay(50);
	}

	if (!memHumanForEval) {
		if (life1>0.0 && life2>0.0) cout << endl << "TIME OVER" << endl;
		if (life1>life2) cout << endl << "player 1 wins" << endl;
		else if (life2>life1) cout << endl << "player 2 wins" << endl;
		else cout << endl << "draw" << endl;
	}
	return 0;
}
