#include "Mutate.h"

/* Add Gaussian noise to linkweights either GAUSSIAN or COLDGAUSSIAN (from zero) */
/* COLDGAUSSIAN means connection weight will be given completely new value */
void mutate_link_weights(Organism* org) {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;
	double randnum,gausspoint,coldgausspoint,age_factor;

	bool severe;  //Once in a while really shake things up
	if (randfloat()<0.5) severe=true;
	else severe=false;

	//Go through all the Genes and perturb their link's weights
	// Loop on all hidden neurons
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			// age factor to mutate new links with increased probability
			if (org->link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*curlink)->innovation_num+1)/((org->link_innovations.back())->innovation_num+1);

			//The following if determines the probabilities of doing cold gaussian
			//mutation, meaning the probability of replacing a link weight with
			//another, entirely random weight.  It is meant to bias such mutations
			//to the tail of a genome, because that is where less time-tested genes
			//reside. The gausspoint and coldgausspoint represent values below
			//which a random float will signify that kind of mutation.  
			if (severe) {
				gausspoint=age_factor;
				coldgausspoint=NEAT::gausspoint_thresh*age_factor;
			}
			else {
				gausspoint=NEAT::gausspoint_thresh*age_factor;
				coldgausspoint=NEAT::coldgausspoint_thresh*age_factor;
			}

			randnum=randfloat();
			if (randnum<coldgausspoint)
				((*curlink)->weight)=NEAT::weight_mut_power*randposneg()*randfloat()*(*curlink)->innovation_num;
			else if (randnum<gausspoint)
				((*curlink)->weight)+=NEAT::weight_mut_power*randposneg()*randfloat()*age_factor;
		}
	}
	// Loop on output neurons (same job)
	for(curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			if (org->link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*curlink)->innovation_num+1)/((org->link_innovations.back())->innovation_num+1);

			if (severe) {
				gausspoint=age_factor;
				coldgausspoint=NEAT::gausspoint_thresh*age_factor;
			}
			else {
				gausspoint=NEAT::gausspoint_thresh*age_factor;
				coldgausspoint=NEAT::coldgausspoint_thresh*age_factor;
			}

			randnum=randfloat();
			if (randnum<coldgausspoint)
				((*curlink)->weight)=NEAT::weight_mut_power*randposneg()*randfloat()*(*curlink)->innovation_num;
			else if (randnum<gausspoint)
				((*curlink)->weight)+=NEAT::weight_mut_power*randposneg()*randfloat()*age_factor;
		}
	}
}

/* Change link's weight totally random (for initialization) */
void randomize_link_weights(Organism* org) {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;

	// Loop on all hidden neurons
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			(*curlink)->weight = randposneg()*randfloat();
		}
	}
	// Loop on output neurons (same job)
	for(curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			(*curlink)->weight = randposneg()*randfloat();
		}
	}
}

/* Toggle genes from enable on to enable off or vice versa. */
void mutate_toggle_enable(Organism* org,int times) {
	list<Neuron*>::iterator curnode,checknode;
	list<IncomingLink*>::iterator curlink,toggle_link; //Gene to toggle
	int randnum,counter;
	bool foundLink;
	double age_factor;

	for (int i=times-1; i>=0; i--) {
		// first decide if toggle on hidden or output neurons
		if (randfloat()<(double)(org->hidden_nodes.size())/(org->hidden_nodes.size()+org->outputs.size())) {
			//Choose a random node
			randnum=randint(0,org->hidden_nodes.size()-1);
			curnode=org->hidden_nodes.begin();
			for (counter=randnum-1; counter>=0; counter--) curnode++;
			// choose a random link if not just one link enabled at this node (to avoid network cuts)
			counter=0;
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if ((*curlink)->enable) counter++;
			}
			if (counter<=1) continue;
			randnum=randint(0,(*curnode)->incoming.size()-1);
			toggle_link=(*curnode)->incoming.begin();
			for (counter=0; counter<randnum; counter++) toggle_link++;
			if (org->link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*toggle_link)->innovation_num+1)/((org->link_innovations.back())->innovation_num+1);

			//Toggle the enable on this gene
			if ((*toggle_link)->enable && randfloat()<age_factor) {
				//We need to make sure that another gene connects out of the in-node
				//Because if not a section of network will break off and become isolated
				for (checknode=org->hidden_nodes.begin(); checknode!=org->hidden_nodes.end(); checknode++) {
					if (checknode==curnode) continue;
					foundLink=false;
					for (curlink=(*checknode)->incoming.begin(); curlink!=(*checknode)->incoming.end(); curlink++) {
						if ((*curlink)->enable && (*curlink)->node->id==(*toggle_link)->node->id) {
							foundLink=true;
							break;
						}
					}
					if (foundLink) break;
				}
				//Disable the gene if it's safe to do so
				if (checknode!=org->hidden_nodes.end()) (*toggle_link)->enable=false;
			}
			else (*toggle_link)->enable=true;
		}
		else {
			//Choose a random node
			randnum=randint(0,org->outputs.size()-1);
			curnode=org->outputs.begin();
			for (counter=randnum-1; counter>=0; counter--) curnode++;
			// choose a random link if not just one link enabled at this node (to avoid network cuts)
			counter=0;
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if ((*curlink)->enable) counter++;
			}
			if (counter<=1) continue;
			randnum=randint(0,(*curnode)->incoming.size()-1);
			toggle_link=(*curnode)->incoming.begin();
			for (counter=0; counter<randnum; counter++) toggle_link++;
			if (org->link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*toggle_link)->innovation_num+1)/((org->link_innovations.back())->innovation_num+1);

			//Toggle the enable on this gene
			if ((*toggle_link)->enable && randfloat()<age_factor) {
				//We need to make sure that another gene connects out of the in-node
				//Because if not a section of network will break off and become isolated
				for (checknode=org->outputs.begin(); checknode!=org->outputs.end(); checknode++) {
					if (checknode==curnode) continue;
					foundLink=false;
					for (curlink=(*checknode)->incoming.begin(); curlink!=(*checknode)->incoming.end(); curlink++) {
						if ((*curlink)->enable && (*curlink)->node->id==(*toggle_link)->node->id) {
							foundLink=true;
							break;
						}
					}
					if (foundLink) break;
				}
				//Disable the gene if it's safe to do so
				if (checknode!=org->outputs.end()) (*toggle_link)->enable=false;
			}
			else (*toggle_link)->enable=true;
		}
	}
}

/* Find first disabled gene and enable it (if any is disabled) */
void mutate_gene_reenable(Organism* org) {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink; //Gene to enable

	// decide which set of neurons to look first
	if (randfloat()<(double)(org->hidden_nodes.size())/(org->hidden_nodes.size()+org->outputs.size())) {
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
		for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
	}
	else {
		for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
	}
}

/* mutate randomly (slightly) one random node threshold */
void mutate_threshold(Organism* org,int times) {
	list<Neuron*>::iterator curnode;
	double randf;
	int randnum;

	for (int i=times-1; i>=0; i--) {
		randf=NEAT::thresh_mut_power*randfloat();
		// decide which set of neurons to attack
		if (randfloat()<(double)(org->hidden_nodes.size())/(org->hidden_nodes.size()+org->outputs.size())) {
			randnum=randint(0,org->hidden_nodes.size()-1);
			curnode=org->hidden_nodes.begin(); 
			for (int i=randnum-1; i>=0; i--) curnode++;
			if (randbit()) {
				if ((*curnode)->threshold+randf<1) (*curnode)->threshold+=randf;
				else (*curnode)->threshold-=randf;
			}
			else {
				if ((*curnode)->threshold-randf>=0.0) (*curnode)->threshold-=randf;
				else (*curnode)->threshold+=randf;
			}
		}
		else {
			randnum=randint(0,org->outputs.size()-1);
			curnode=org->outputs.begin(); 
			for (int i=randnum-1; i>=0; i--) curnode++;
			if (randbit()) {
				if ((*curnode)->threshold+randf<1) (*curnode)->threshold+=randf;
				else (*curnode)->threshold-=randf;
			}
			else {
				if ((*curnode)->threshold-randf>=0.0) (*curnode)->threshold-=randf;
				else (*curnode)->threshold+=randf;
			}
		}
	}
}

/* Mutate the genome by adding a new link between 2 random Neurons */
/* If the 2 Neurons are already connected, keep trying tries times */
bool mutate_add_link(Organism* org,int times) {
	list<Neuron*>::iterator it_node1,it_node2;
	list<IncomingLink*>::iterator curlink;
	int randnum; //random node number
	double randf;
	
	for (int i=times-1; i>=0; i--) {
		// randomly choose a pair of nodes (not allow connection inp-inp or out-out)
		randf=randfloat();
		bool firstIsInput=false,secondIsInput=false,firstIsOutput=false;
		if (randf<(double)org->hidden_nodes.size()/(org->inputs.size()+org->hidden_nodes.size()+org->outputs.size())) {
			randnum=randint(0,org->hidden_nodes.size()-1);
			it_node1=org->hidden_nodes.begin();
		}
		else if (randf<(double)(org->hidden_nodes.size()+org->outputs.size())
					/(org->inputs.size()+org->hidden_nodes.size()+org->outputs.size())) {
			randnum=randint(0,org->outputs.size()-1);
			it_node1=org->outputs.begin();
			firstIsOutput=true;
		}
		else {
			randnum=randint(0,org->inputs.size()-1);
			it_node1=org->inputs.begin();
			firstIsInput=true;
		}
		while (--randnum>=0) it_node1++;
		randf=randfloat();
		if (firstIsInput) {
			if (randf<(double)org->hidden_nodes.size()/(org->hidden_nodes.size()+org->outputs.size())) {
				randnum=randint(0,org->hidden_nodes.size()-1);
				it_node2=org->hidden_nodes.begin();
			}
			else {
				randnum=randint(0,org->outputs.size()-1);
				it_node2=org->outputs.begin();
			}
		}
		else if (firstIsOutput) {
			if (randf<(double)org->hidden_nodes.size()/(org->hidden_nodes.size()+org->inputs.size())) {
				randnum=randint(0,org->hidden_nodes.size()-1);
				it_node2=org->hidden_nodes.begin();
			}
			else {
				randnum=randint(0,org->inputs.size()-1);
				it_node2=org->inputs.begin();
			}
		}
		else {
			if (randf<(double)org->hidden_nodes.size()/(org->inputs.size()+org->hidden_nodes.size()+org->outputs.size())) {
				randnum=randint(0,org->hidden_nodes.size()-1);
				it_node2=org->hidden_nodes.begin();
			}
			else if (randf<(double)(org->hidden_nodes.size()+org->outputs.size())
						/(org->inputs.size()+org->hidden_nodes.size()+org->outputs.size())) {
				randnum=randint(0,org->outputs.size()-1);
				it_node2=org->outputs.begin();
			}
			else {
				randnum=randint(0,org->inputs.size()-1);
				it_node2=org->inputs.begin();
				secondIsInput=true;
			}
		}
		while (--randnum>=0) it_node2++;

		// order the iterators to have "it1<it2"
		list<Neuron*>::iterator aux_it;
		if (secondIsInput || firstIsOutput) {
			aux_it=it_node1;
			it_node1=it_node2;
			it_node2=aux_it;
		}
		
		// check if they are connected AND different (!)
		// NOTE: we don't allow double sense connections; perhaps could be useful,
		// but it seems we can do without (and final interpretation will be much simpler)
		if ((*it_node1)->id==(*it_node2)->id) continue;
		curlink=(*it_node2)->incoming.begin();
		while (curlink!=(*it_node2)->incoming.end() && (*curlink)->node->id!=(*it_node1)->id) curlink++;
		if (curlink!=(*it_node2)->incoming.end()) continue;
		curlink=(*it_node1)->incoming.begin();
		while (curlink!=(*it_node1)->incoming.end() && (*curlink)->node->id!=(*it_node2)->id) curlink++;
		if (curlink!=(*it_node1)->incoming.end()) continue;

		// they are not connected, so we create the link with weight 1.0
		// search if this mutation already exists
		int index=global_link_innovation.size()-1;
		while (index>=0 && 
			(global_link_innovation[index]->incnode->id!=(*it_node1)->id || global_link_innovation[index]->outnode->id!=(*it_node2)->id))
			index--;
		if (index>=0) {
			// mutation ever met:
			org->add_link_innovation(global_link_innovation[index]);
			(*it_node2)->add_incoming(*it_node1,1.0,global_link_innovation[index]->innovation_num);
		}
		else {
			// new mutation
			(*it_node2)->add_incoming(*it_node1,1.0,cur_innov);
			Link* lk=new Link(*it_node1,*it_node2,cur_innov++);
			org->link_innovations.push_back(lk);
			global_link_innovation.push_back(lk);
		}
		org->nb_links++;
		return true;
	}
	return false;
}

//This mutator adds a node to an organism by inserting it in the middle
//of an existing link between 2 nodes. This broken link will be disabled
//and now represented by 2 links with the new node between them.
//The global innovs list is used to compare the innovation
//with other innovations in the list and see whether they match.
//If they do, the same innovation number will be assigned to the new
//genes. If a disabled link is chosen after times trials, 
//then the method just exits with false
bool mutate_add_node(Organism* org,int times) {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink; //random gene containing the original link
	int randnum; //The random gene number

	for (int i=times-1; i>=0; i--) {
		// pseudo-randomly choose a link
		if (randfloat()<(double)org->hidden_nodes.size()/(org->hidden_nodes.size()+org->outputs.size())) {
			randnum=randint(0,org->hidden_nodes.size()-1);
			curnode=org->hidden_nodes.begin();
		}
		else {
			randnum=randint(0,org->outputs.size()-1);
			curnode=org->outputs.begin();
		}
		while (--randnum>=0) curnode++;
		curlink=(*curnode)->incoming.begin();
		randnum=randint(0,(*curnode)->incoming.size()-1);
		while (--randnum>=0) curlink++;

		// NOTE: for such a mutation, the link can be disable;
		// in this case, the mutation will be active later
		if (randfloat()<(double)((*curlink)->innovation_num+2)/(cur_innov+1)) {
			// search if this mutation already exists
			int index=global_node_innovation.size()-1;
			while (global_node_innovation[index]!=0 && 
				(global_node_innovation[index]->incnode->id!=(*curlink)->node->id || global_node_innovation[index]->outnode->id!=(*curnode)->id))
				index--;
			if (global_node_innovation[index]!=0) {
				// mutation ever met: 
				//WARNING we have to check if this mutation didn't occur in this organism's history!
				list<LinkAndNode*>::iterator check_exists;
				for (check_exists=org->node_innovations.begin(); check_exists!=org->node_innovations.end(); check_exists++) {
					if ((*check_exists)->midnode->id==global_node_innovation[index]->midnode->id) break;
				}
				if (check_exists!=org->node_innovations.end()) continue;
				org->add_node_innovation(global_node_innovation[index]);
				Neuron* newnode=new Neuron(global_node_innovation[index]->midnode->id,
											global_node_innovation[index]->midnode->functype,
											global_node_innovation[index]->midnode->threshold);
				newnode->add_incoming((*curlink)->node,(*curlink)->weight,global_node_innovation[index]->innovation_num);
				(*curnode)->add_incoming(newnode,(*curlink)->weight,global_node_innovation[index]->innovation_num+1);
				(*curlink)->enable=false;
				org->add_neuron(newnode);
			}
			else {
				// new mutation: we create the new neuron with SIGMOID activation type;
				// LINEAR seems to not work very well; if so, to get random activation type
				// replace 0 with: randint(0,nbActTypes-1); then we could also mutate activation types.
				// initial threshold is 0: the neuron is sensitive to all incomings.
				Neuron* newnode=new Neuron(cur_node_id++,0,0.0);
				newnode->add_incoming((*curlink)->node,(*curlink)->weight,cur_innov);

				// add the first new link mutation:
				Link* lk=new Link((*curlink)->node,newnode,cur_innov++);
				org->link_innovations.push_back(lk);
				global_link_innovation.push_back(lk);
				(*curnode)->add_incoming(newnode,(*curlink)->weight,cur_innov);

				// add the second new link mutation:
				lk=new Link(newnode,*curnode,cur_innov++);
				org->link_innovations.push_back(lk);
				global_link_innovation.push_back(lk);
				
				// finally, add the new node innovation:
				LinkAndNode* lkn=new LinkAndNode((*curlink)->node,newnode,*curnode,cur_innov-2);
				org->node_innovations.push_back(lkn);
				global_node_innovation.push_back(lkn);
				(*curlink)->enable=false;
				org->add_neuron(newnode);
			}
			org->nb_links+=2; // two new links appeared
			return true;
		}
	}
	return false;
}
