#include "Organism.h"

//Constructor which spawns off an input file, with weights random or given by user.
// In what follows:
// zero_threshold is a boolean telling if the node's thresholds are zero or given by user,
// random_weights is a boolean telling if the link's weights are given by user.
/* (WARNING: inside each node's group, id's must be in increasing order;
 * AND: highest ID's must be in the hidden layers); file format:
 * genomestart <zero_threshold> <random_weights>
 * incnode <id>
 * incnode <id>
 * ..etc
 * [ optional:
 * midnode <id> <functype> [<threshold>] <incoming_id1> <innov_num1> [<weight1>] ... <incoming_idn> <innov_numn> [<weightn>] ; 
 * midnode <id> <functype> [<threshold>] <incoming_id1> <innov_num1> [<weight1>] ... <incoming_idn> <innov_numn> [<weightn>] ; 
 * ..etc ]
 * outnode <id> <functype> [<threshold>] <incoming_id1> <innov_num1> [<weight1>] ... <incoming_idn> <innov_numn> [<weightn>] ;
 * outnode <id> <functype> [<threshold>] <incoming_id1> <innov_num1> [<weightn>] ... <incoming_idn> <innov_numn> [<weightn>] ;
 * ..etc
 * [ optional, depending if hidden_nodes empty or not:
 * NOTE: we could also infer them, giving arbitrary innovation numbers,
 * but as the mutations depend on innovation numbers it could be interesting to enter them:
 * link_innovations <id1> <id2> <innov_num> <id1> <id2> <innov_num> ..etc ;
 * node_innovations <id1> <id2> <id3> <id1> <id2> <id3> ..etc ; ]
 * genomeend */
Organism::Organism (ifstream& iFile) {
	char curword[20];
	int node_id,node_id1,node_id2,node_id3,ft,innov_num;
	double weight=0.0,thresh=0.0;
	Neuron *newnode=0;
	list<Neuron*>::iterator curnode;
	vector<Neuron*> nodes; // stores nodes at the first pass
	Neuron* fill_node=0; // to fill empty spaces in nodes vector
	nb_links=0;

	iFile >> curword;
	if (strcmp(curword,"genomestart")==0) iFile >> curword;
	bool zero_threshold=(curword[0]!='0');
	iFile >> curword;
	bool random_weights=(curword[0]!='0');
	id=cur_org_id++;

	bool done=false;
	//Loop until file is finished, parsing each line
	while (!done) {
		iFile >> curword;
		//Check for end of Genome
		if (strcmp(curword,"genomeend")==0) done=true;

		//Ignore comments surrounded by /* */
		else if (strcmp(curword,"/*")==0) {
			iFile >> curword;
			while (strcmp(curword,"*/")!=0) iFile >> curword;
		}

		// Read in an input node
		else if (strcmp(curword,"incnode")==0) {
			iFile >> curword;
			node_id=atoi(curword);
			//Allocate the new node
			newnode=new Neuron(node_id);
			if (cur_node_id<=node_id) cur_node_id=node_id+1; // ensure node id's will be consistent
			// push its reference in a vector, sized to fit node's id
			if ((int)nodes.size()<=node_id) {
				for (int i=node_id-nodes.size()-1; i>=0; i--) nodes.push_back(fill_node);
				nodes.push_back(newnode);
			}
			else nodes[node_id]=newnode;
			//Add the node to the list of nodes
			inputs.push_back(newnode);
		}
		// Read in an hidden node
		else if (strcmp(curword,"midnode")==0) {
			iFile >> curword;
			node_id=atoi(curword);
			iFile >> curword;
			ft=atoi(curword);
			if (zero_threshold) thresh=0.0;
			else {
				iFile >> curword;
				thresh=atof(curword);
			}

			// size nodes to fit node's id:
			if ((int)nodes.size()<=node_id) {
				for (int i=node_id-nodes.size(); i>=0; i--) nodes.push_back(fill_node);
			}
			//Allocate the new node if needed
			if (nodes[node_id]!=0) {
				newnode=nodes[node_id];
				newnode->functype=ft;
				newnode->threshold=thresh;
			}
			else {
				newnode=new Neuron(node_id,ft,thresh);
				nodes[node_id]=newnode;
			}
			if (cur_node_id<=node_id) cur_node_id=node_id+1; // ensure node id's will be consistent

			iFile >> curword;
			while (strcmp(curword,";")!=0) {
				node_id=atoi(curword);
				iFile >> curword;
				innov_num=atoi(curword);
				if (random_weights) weight=randposneg()*randfloat();
				else {
					iFile >> curword;
					weight=atof(curword);
				}

				// size nodes to fit node's id:
				if ((int)nodes.size()<=node_id) {
					for (int i=node_id-nodes.size(); i>=0; i--) nodes.push_back(fill_node);
				}
				// two cases: nodes[node_id] initialized or not
				if (nodes[node_id]!=0)
					newnode->add_incoming(nodes[node_id],weight,innov_num);
				else {
					nodes[node_id]=new Neuron(node_id); // ft and thresh will be set later
					newnode->add_incoming(nodes[node_id],weight,innov_num);
				}
				iFile >> curword;
			}
			//Add the node to the list of nodes
			hidden_nodes.push_back(newnode);
		}
		// Read in an output node (can't connect with another output, so this code seems safe)
		else if (strcmp(curword,"outnode")==0) {
			iFile >> curword;
			node_id=atoi(curword);
			iFile >> curword;
			ft=atoi(curword);
			if (zero_threshold) thresh=0.0;
			else {
				iFile >> curword;
				thresh=atof(curword);
			}

			//Allocate the new node
			newnode=new Neuron(node_id,ft,thresh);
			// size nodes to fit node's id:
			if ((int)nodes.size()<=node_id) {
				for (int i=node_id-nodes.size(); i>=0; i--) nodes.push_back(fill_node);
			}

			if (cur_node_id<=node_id) cur_node_id=node_id+1; // ensure node id's will be consistent
			iFile >> curword;
			while (strcmp(curword,";")!=0) {
				node_id=atoi(curword);
				iFile >> curword;
				innov_num=atoi(curword);
				if (random_weights) weight=randposneg()*randfloat();
				else {
					iFile >> curword;
					weight=atof(curword);
				}
				newnode->add_incoming(nodes[node_id],weight,innov_num);
				iFile >> curword;
			}
			//Add the node to the list of nodes and to the nodes tab:
			outputs.push_back(newnode);
			nodes[newnode->id]=newnode;
		}
		// Read link innovations, check if they exists at global
		else if (strcmp(curword,"link_innovations")==0) {
			iFile >> curword;
			while (strcmp(curword,";")!=0) {
				node_id1=atoi(curword);
				iFile >> curword;
				node_id2=atoi(curword);
				iFile >> curword;
				innov_num=atoi(curword);
				int i;
				for (i=global_link_innovation.size()-1; i>=0; i--) {
					if (global_link_innovation[i]->innovation_num==innov_num) break;
				}
				if (i>=0) link_innovations.push_back(global_link_innovation[i]);
				else {
					Link* lk=new Link(new Neuron(node_id1,nodes[node_id1]->functype,nodes[node_id1]->threshold),
									new Neuron(node_id2,nodes[node_id2]->functype,nodes[node_id2]->threshold),innov_num);
					global_link_innovation.push_back(lk);
					link_innovations.push_back(lk);
				}
				// adjust cur_innov to be consistent:
				if ((link_innovations.back())->innovation_num>=cur_innov) cur_innov=(link_innovations.back())->innovation_num+1;
				iFile >> curword;
			}
		}
		// Read node innovations, check if they exists at global
		else if (strcmp(curword,"node_innovations")==0) {
			// don't forget to initialise the array of innovations
			// we do that because we want to access the innovations with instructions like:
			// global_node_innovation[node->id], but of course the initial nodes don't
			// correspond to some innovations, and the corresponding elements inside global_node_innovation
			// will never be accessed (if so, there is a problem..)
			if (global_node_innovation.empty()) {
				LinkAndNode* fill_innov=0;
				for (int i=inputs.size()+hidden_nodes.size()+outputs.size()-1; i>=0; i--) global_node_innovation.push_back(fill_innov);
			}
			iFile >> curword;
			while (strcmp(curword,";")!=0) {
				node_id1=atoi(curword);
				iFile >> curword;
				node_id2=atoi(curword);
				iFile >> curword;
				node_id3=atoi(curword);
				int i;
				for (i=global_node_innovation.size()-1; i>=0 && global_node_innovation[i]!=0; i--) {
					if (global_node_innovation[i]->midnode->id==node_id2) break;
				}
				if (global_node_innovation[i]!=0) node_innovations.push_back(global_node_innovation[i]);
				else {
					// we must find the link innovation num between node1 and node2:
					list<Link*>::iterator curlk;
					for (curlk=link_innovations.begin(); curlk!=link_innovations.end() 
								&& ((*curlk)->incnode->id!=node_id1 || (*curlk)->outnode->id!=node_id2); curlk++) {}
					LinkAndNode* ln=new LinkAndNode(new Neuron(node_id1,nodes[node_id1]->functype,nodes[node_id1]->threshold),
													new Neuron(node_id2,nodes[node_id2]->functype,nodes[node_id2]->threshold),
													new Neuron(node_id3,nodes[node_id3]->functype,nodes[node_id3]->threshold),
													(*curlk)->innovation_num);
					global_node_innovation.push_back(ln);
					node_innovations.push_back(ln);
				}
				iFile >> curword;
			}
		}
	}
	fitness=0.0;
	generation=0;
	eliminate=false;
	winner=false;

	// don't forget to initialise the array of innovations
	// we do that because we want to access the innovations with instructions like:
	// global_node_innovation[node->id], but of course the initial nodes don't
	// correspond to some innovations, and the corresponding elements inside global_node_innovation
	// will never be accessed (if so, there is a problem..)
	if (global_node_innovation.empty()) {
		LinkAndNode* fill_innov=0;
		for (int i=inputs.size()+hidden_nodes.size()+outputs.size()-1; i>=0; i--) global_node_innovation.push_back(fill_innov);
	}
}

// copy constructor
Organism::Organism (const Organism* org) {
	// preprocessing: duplicate all the nodes, without links
	list<Neuron*>::const_iterator curnode;
	list<IncomingLink*>::const_iterator curlink;
	int sizeID=0;
	// highest ID's are typically found in the hidden layers;
	// if no hidden layer, we go through outputs neurons
	if (!org->hidden_nodes.empty()) {
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	else {
		for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	Neuron** tabNodes=new Neuron*[sizeID+1];

	// duplication phase
	for (curnode=org->inputs.begin(); curnode!=org->inputs.end(); curnode++)
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
	for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++)
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++)
		tabNodes[(*curnode)->id]=new Neuron(*curnode);

	// add the relations between nodes, one by one, and then add all nodes to the lists
	for (curnode=org->inputs.begin(); curnode!=org->inputs.end(); curnode++)
		inputs.push_back(tabNodes[(*curnode)->id]);
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
			tabNodes[(*curnode)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
		hidden_nodes.push_back(tabNodes[(*curnode)->id]);
	}
	for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
			tabNodes[(*curnode)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
		outputs.push_back(tabNodes[(*curnode)->id]);
	}

	//copy innovation's lists
	list<Link*>::const_iterator curlk;
	list<LinkAndNode*>::const_iterator curln;
	for (curlk=org->link_innovations.begin(); curlk!=org->link_innovations.end(); curlk++)
		link_innovations.push_back(*curlk);
	for (curln=org->node_innovations.begin(); curln!=org->node_innovations.end(); curln++)
		node_innovations.push_back(*curln);

	//copy other fields and calculate id
	id=cur_org_id++;
	nb_links=org->nb_links;
	generation=org->generation;
	fitness=org->fitness;
	eliminate=false;
	winner=false;
}

//Activates the net such (maybe all outputs will not be active if some reproduction has been really unlucky;
// in this case the output is simply activation_function(0))
void Organism::activate() {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;
	/* For each node, compute the sum of its incoming activations */
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		(*curnode)->activeIncoming=false;
		(*curnode)->incoming_sum=0.0;
		/* For each incoming connection, add the activity from the connection to the activesum */
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			if ((*curlink)->enable && (*curlink)->node->enable) {
				(*curnode)->incoming_sum+=((*curlink)->weight)*((*curlink)->node->activation);
				if (!((*curnode)->activeIncoming)) (*curnode)->activeIncoming=true;
			}
		}
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		(*curnode)->activeIncoming=false;
		(*curnode)->incoming_sum=0.0;
		/* For each incoming connection, add the activity from the connection to the activesum */
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			if ((*curlink)->enable && (*curlink)->node->enable) {
				(*curnode)->incoming_sum+=((*curlink)->weight)*((*curlink)->node->activation);
				if (!((*curnode)->activeIncoming)) (*curnode)->activeIncoming=true;
			}
		}
	}
	/* Now activate all the non-sensor nodes off their incoming activation */
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		if (!((*curnode)->activeIncoming)) {
			(*curnode)->activation=0.0;
			continue;
		}
		if ((*curnode)->functype==SIGMOID) {
			//Sigmoidal activation; see comments of fsigmoid in Functions.h
			(*curnode)->activation=1.0/(1.0+(exp(-(*curnode)->incoming_sum)));
			// no activation if under threshold
			if ((*curnode)->activation<(*curnode)->threshold) (*curnode)->activation=0.0;
		}
		else if ((*curnode)->functype==LINEAR) {
			// simple linear activation
			(*curnode)->activation=(*curnode)->incoming_sum;
			// no activation if under threshold
			if ((*curnode)->activation<(*curnode)->threshold) (*curnode)->activation=0.0;
		}
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		if (!((*curnode)->activeIncoming)) {
			(*curnode)->activation=0.0;
			continue;
		}
		if ((*curnode)->functype==SIGMOID) {
			//Sigmoidal activation; see comments of fsigmoid in Functions.h
			(*curnode)->activation=1.0/(1.0+(exp(-(*curnode)->incoming_sum)));
			// no activation if under threshold
			if ((*curnode)->activation<(*curnode)->threshold) (*curnode)->activation=0.0;
		}
		else if ((*curnode)->functype==LINEAR) {
			// simple linear activation
			(*curnode)->activation=(*curnode)->incoming_sum;
			// no activation if under threshold
			if ((*curnode)->activation<(*curnode)->threshold) (*curnode)->activation=0.0;
		}
	}
}

// useful for pig and fight games: gives the most active output;
// select at random if more than one maxima.
int Organism::getActiveOutput() {
	// stores the outputs to fasten the search for values:
	double outs[outputs.size()];
	int i=0;
	for (list<Neuron*>::iterator curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		outs[i]=(*curnode)->activation;
		i++;
	}

	// search max's output(s):
	vector<int> maxima;
	maxima.push_back(outputs.size()-1);
	double maxi=outs[outputs.size()-1];
	for (int i=outputs.size()-2; i>=0; i--) {
		if (outs[i]>maxi) {
			maxima.clear();
			maxima.push_back(i);
			maxi=outs[i];
		}
		else if (outs[i]==maxi) maxima.push_back(i);
	}

	if (maxima.size()==1) return maxima[0];
	return maxima[randint(0,maxima.size()-1)];
}

/* This function gives a measure of compatibility between
     two Organisms by computing a linear combination of 3
     characterizing variables of their compatibilty.
     The 3 variables represent PERCENT DISJOINT GENES,
     PERCENT EXCESS GENES, WEIGHT DIFFERENCE WITHIN
     MATCHING GENES.  So the formula for compatibility
     is inversly proportional to:
	 disjoint_coeff*pdg+excess_coeff*peg+weight_diff_coeff*mdmg.
     The 3 coefficients are global system parameters */
double Organism::compatibility(const Organism* org) const {
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink,curlink1,curlink2;

	//Set up the counters
	int num_disjoint=0;
	int num_excess=0;
	double weight_diff=0.0;
	int num_matching=0;  //Used to normalize mutation_num differences
	
	//Now move through the Genes of each parent until both genomes end;
	//neurons are order by increasing id and innovations by increasing innov_num,
	//so it's relatively easy
	curnode1=hidden_nodes.begin();
	curnode2=(org->hidden_nodes).begin();
	while (curnode1!=hidden_nodes.end() || curnode2!=(org->hidden_nodes).end()) {
		if (curnode1==hidden_nodes.end()) {
			num_excess+=(*curnode2)->incoming.size();
			curnode2++;
		}
		else if (curnode2==(org->hidden_nodes).end()) {
			num_excess+=(*curnode1)->incoming.size();
			curnode1++;
		}
		else {
			//two cases: features in both organisms, or just in one
			if ((*curnode1)->id==(*curnode2)->id) {
				curlink1=(*curnode1)->incoming.begin();
				curlink2=(*curnode2)->incoming.begin();
				while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
					if (curlink1==(*curnode1)->incoming.end()) {
						num_disjoint++;
						curlink2++;
					}
					else if (curlink2==(*curnode2)->incoming.end()) {
						num_disjoint++;
						curlink1++;
					}
					else {
						// two cases: the links point the same node, or not
						if ((*curlink1)->node->id<(*curlink2)->node->id) {
							num_disjoint++;
							curlink1++;
						}
						else if ((*curlink1)->node->id>(*curlink2)->node->id) {
							num_disjoint++;
							curlink2++;
						}
						else {
							num_matching++;
							weight_diff+=fabs((*curlink1)->weight-(*curlink2)->weight);
							curlink1++;
							curlink2++;
						}
					}
				}
				curnode1++;
				curnode2++;
			}
			else if ((*curnode1)->id<(*curnode2)->id) {
				num_disjoint+=(*curnode1)->incoming.size();
				curnode1++;
			}
			else {
				num_disjoint+=(*curnode2)->incoming.size();
				curnode2++;
			}
		}
	}
	// then look at the output nodes
	curnode1=outputs.begin();
	curnode2=(org->outputs).begin();
	while (curnode1!=outputs.end()) {
		curlink1=(*curnode1)->incoming.begin();
		curlink2=(*curnode2)->incoming.begin();
		while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
			if (curlink1==(*curnode1)->incoming.end()) {
				num_disjoint++;
				curlink2++;
			}
			else if (curlink2==(*curnode2)->incoming.end()) {
				num_disjoint++;
				curlink1++;
			}
			else {
				// two cases: the links point the same node, or not
				if ((*curlink1)->node->id<(*curlink2)->node->id) {
					num_disjoint++;
					curlink1++;
				}
				else if ((*curlink1)->node->id>(*curlink2)->node->id) {
					num_disjoint++;
					curlink2++;
				}
				else {
					num_matching++;
					weight_diff+=fabs((*curlink1)->weight-(*curlink2)->weight);
					curlink1++;
					curlink2++;
				}
			}
		}
		curnode1++;
		curnode2++;
	}

	// finally, calculate the formula
	return NEAT::disjoint_coeff*num_disjoint+NEAT::excess_coeff*num_excess+NEAT::weight_diff_coeff*((weight_diff+1.0)/(num_matching+1));
}

// function to reorganize the network in layers; return max_weight_link
double Organism::arrange_topology(list<list<Neuron*> >& n_layers) const {
	list<Neuron*>::const_iterator curnode;
	list<IncomingLink*>::const_iterator curlink;
	// idea: neurons are sorted by layers, the outputs first;
	// we ignore loops which come back to previously marked nodes
	list< list<Neuron*> >::iterator layers_iter;
	// initialization: visited=false for each neuron, and give them a label (input, output or empty)
	for (curnode=inputs.begin(); curnode!=inputs.end(); curnode++) {
		(*curnode)->visited=false;
		(*curnode)->label="input";
	}
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		(*curnode)->visited=false;
		(*curnode)->label="";
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		(*curnode)->visited=false;
		(*curnode)->label="output";
	}
	
	// initialization of the layers with the outputs:
	list<Neuron*> layer;
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		(*curnode)->visited=true;
		layer.push_back(*curnode);
	}
	n_layers.push_back(layer);
	double max_weight_link=0.0;
	// loop: while some neurons are missing, go on
	while (true) {
		layer.clear();
		for (curnode=n_layers.back().begin(); curnode!=n_layers.back().end(); curnode++) {
			if ((*curnode)->label=="input") continue; // inputs are treated at the end
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->node->visited && (*curlink)->node->label!="input") {
					(*curlink)->node->visited=true;
					layer.push_back((*curlink)->node);
				}
				if (fabs((*curlink)->weight)>max_weight_link) max_weight_link=fabs((*curlink)->weight);
			}
		}
		if (layer.empty()) break;
		n_layers.push_back(layer);
	}

	list<Neuron*>::iterator curnode_,checknode,pushednode;
	list<IncomingLink*>::iterator curlink_;
	bool rearranged=false,foundLink;
	// then rearrange the layers to get rid of the horizontal connections
	while (!rearranged) {
		rearranged=true;
		layers_iter=n_layers.begin();
		while (layers_iter!=n_layers.end()) {
			layer.clear();
			curnode_=(*layers_iter).begin();
			while (curnode_!=(*layers_iter).end()) {
				foundLink=false;
				// check if this node connects one of the visited nodes from the layer in exploration
				// if yes, we create a new layer for him and his homologues
				for (curlink_=(*curnode_)->incoming.begin(); curlink_!=(*curnode_)->incoming.end(); curlink_++) {
					checknode=(*layers_iter).begin();
					while (checknode!=(*layers_iter).end()) {
						if ((*checknode)->id==(*curlink_)->node->id) {
							layer.push_back(*curnode_);
							// erase the node pushed back to the next layer:
							pushednode=curnode_;
							curnode_++;
							(*layers_iter).erase(pushednode);
							foundLink=true;
							break;
						}
						checknode++;
					}
					if (foundLink) break;
				}
				if (!foundLink) curnode_++;
			}
			if (!layer.empty()) {
				rearranged=false;
				layers_iter++;
				n_layers.insert(layers_iter,layer);
			}
			layers_iter++;
		}
	}
	// add the final layer: the inputs
	layer.clear();
	for (curnode=inputs.begin(); curnode!=inputs.end(); curnode++) layer.push_back(*curnode);
	n_layers.push_back(layer);

	// initialize neuron's depths:
	int curDepth=n_layers.size()-1;
	for (layers_iter=n_layers.begin(); layers_iter!=n_layers.end(); layers_iter++) {
		for (curnode_=(*layers_iter).begin(); curnode_!=(*layers_iter).end(); curnode_++) (*curnode_)->depth=curDepth;
		curDepth--;
	}

	return max_weight_link;
}

void Organism::print_to_file(char* filename) {
	ofstream outFile(filename);
	//Make sure it worked
	if (outFile==0) {
		cerr << "Can't open " << filename << " for output" << endl;
		return;
	}
	outFile << "genomestart " << 0 << " " << 0 << endl;
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;
	for (curnode=inputs.begin(); curnode!=inputs.end(); curnode++)
		outFile << "incnode " << (*curnode)->id << endl;
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		outFile << "midnode " << (*curnode)->id << " " << (*curnode)->functype << " " << (*curnode)->threshold << " ";
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
			outFile << (*curlink)->node->id << " " << (*curlink)->innovation_num << " " << (*curlink)->weight << " ";
		outFile << ";" << endl;
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		outFile << "outnode " << (*curnode)->id << " " << (*curnode)->functype << " " << (*curnode)->threshold << " ";
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
			outFile << (*curlink)->node->id << " " << (*curlink)->innovation_num << " " << (*curlink)->weight << " ";
		outFile << ";" << endl;
	}
	if (!link_innovations.empty()) outFile << "link_innovations ";
	for (list<Link*>::iterator curlk=link_innovations.begin(); curlk!=link_innovations.end(); curlk++) {
		outFile << (*curlk)->incnode->id << " " << (*curlk)->outnode->id << " " << (*curlk)->innovation_num << " ";
	}
	if (!link_innovations.empty()) outFile << ";" << endl;
	if (!node_innovations.empty()) outFile << "node_innovations ";
	for (list<LinkAndNode*>::iterator curln=node_innovations.begin(); curln!=node_innovations.end(); curln++) {
		outFile << (*curln)->incnode->id << " " << (*curln)->midnode->id << " " << (*curln)->outnode->id << " ";
	}
	if (!node_innovations.empty()) outFile << ";" << endl;
	outFile << "genomeend" << endl;
}
