#ifndef _Organism
#define _Organism

#include <iostream>
#include <list>
#include <vector>
#include "Neuron.h"
#include "Functions.h"
#include "IDCounters.h"
#include "Innovation.h"
#include "Neat.h"

using namespace std;

/* A NETWORK/Organism is a LIST of input NODEs and a LIST of output NODEs           
   The point of the network is to define a single entity which can evolve
   or learn on its own, even though it may be part of a larger framework */

class Organism {

public:

	// structural members (genotype)
	list<Neuron*> inputs; //Nodes that input into the network
	list<Neuron*> outputs; //Values output by the network
	list<Neuron*> hidden_nodes; // all the hidden neurons in the net (empty for each one at the beginning)
	int id;
	// phenotype:
	double fitness,orig_fitness; //A measure of fitness for the Organism
	int expected_offspring; //Number of children this Organism may have
	int generation; //Tells which generation this Organism is from
	bool eliminate; // true if too bad or too old to survive
	bool winner; // tells if this genome solve the problem
	int nb_links; // number of links (enable or not); useful for mutation and statistics
	list<list<Neuron*> > layers; // organisation of the network in layers,
								// useful for learning and drawing.

	// innovations experienced by this organism
	list<Link*> link_innovations;
	list<LinkAndNode*> node_innovations;

	Organism () {
		id=cur_org_id++;
		generation=0;
		eliminate=false;
		winner=false;
		nb_links=0;
	}

	//Constructor which spawns off an input file, with weights random or given by user.
	Organism(ifstream& iFile);
	
	// copy constructor
	Organism (const Organism* org);
	
	~Organism() {
		/* Erase all neurons from the lists */
		for (list<Neuron*>::iterator curnode=inputs.begin(); curnode!=inputs.end(); curnode++)
			delete *curnode;
		for (list<Neuron*>::iterator curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++)
			delete *curnode;
		for (list<Neuron*>::iterator curnode=outputs.begin(); curnode!=outputs.end(); curnode++)
			delete *curnode;
	}

	//Adds a Neuron in the id's order
	void add_neuron(Neuron* node) {
		list<Neuron*>::iterator curnode=hidden_nodes.begin();
		while (curnode!=hidden_nodes.end() && (*curnode)->id<node->id) curnode++;
		hidden_nodes.insert(curnode,node);
	}
	
	//add innovations in innovation num's order:
	void add_link_innovation(Link* lk) {
		list<Link*>::iterator curlk=link_innovations.begin();
		while (curlk!=link_innovations.end() && (*curlk)->innovation_num<lk->innovation_num) curlk++;
		link_innovations.insert(curlk,lk);
	}
	void add_node_innovation(LinkAndNode* lkn) {
		list<LinkAndNode*>::iterator curln=node_innovations.begin();
		while (curln!=node_innovations.end() && (*curln)->midnode->id<lkn->midnode->id) curln++;
		node_innovations.insert(curln,lkn);
	}

	// reinitialise the network before a new call to load_sensors():
	void flush() {
		list<Neuron*>::iterator curnode;
		for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) (*curnode)->activation=0.0;
		for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) (*curnode)->activation=0.0;
	}

	//Takes an array of sensor values and loads it into SENSOR inputs
	void load_sensors(double* sensvals) {
		int counter=0;
		for (list<Neuron*>::iterator curnode=inputs.begin(); curnode!=inputs.end(); curnode++) {
			if ((*curnode)->enable)
				(*curnode)->activation=sensvals[counter];
			counter++;
		}
	}

	// reinit the inputs to enable:
	void reinitInputs() {
		for (list<Neuron*>::iterator curnode=inputs.begin(); curnode!=inputs.end(); curnode++)
			(*curnode)->enable=true;
	}

	void activate(); //Activates the net such that all outputs are active
	
	// useful for pig and fight games: gives the most active output
	int getActiveOutput();

	/* This function gives a measure of compatibility between
     two Organisms by computing a linear combination of 3
     characterizing variables of their compatibilty.
     The 3 variables represent PERCENT DISJOINT GENES,
     PERCENT EXCESS GENES, WEIGHT DIFFERENCE WITHIN
     MATCHING GENES.  So the formula for compatibility
     is inversly proportional to:
	 disjoint_coeff*pdg+excess_coeff*peg+weight_diff_coeff*mdmg.
     The 3 coefficients are global system parameters */
	double compatibility(const Organism* org) const;

	// function to reorganize the network in layers
	double arrange_topology(list<list<Neuron*> >& n_layers) const;

	bool operator<(const Organism& org) const {
		return fitness > org.fitness;
	}
	
	// print to a file so that the Organism can be retrieved later
	void print_to_file(char* name);

};

#endif
