#ifndef	_PigEngine
#define	_PigEngine

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <cmath>
#include "Dirty.h"
#include "Organism.h"
#include "PigUtiles.h"

/*
 * Engine
 */
PIG_engine *pig_open(SDL_Surface *screen);
void pig_close(PIG_engine *pe);

/* Set viewport size and position */
void pig_viewport(PIG_engine *pe, int x, int y, int w, int h);

/* Start engine at logic time 'frame' */
void pig_start(PIG_engine *pe, int frame);

void run_timers(PIG_engine *pe, PIG_object *po);

void test_offscreen(PIG_engine *pe, PIG_object *po, PIG_sprite *s);

void sprite_sprite_one(PIG_object *po, PIG_object *po2, float t, float hitdist);

void test_sprite_sprite(PIG_engine *pe, PIG_object *po, PIG_sprite *s);

void test_sprite_map(PIG_engine *pe, PIG_object *po, PIG_sprite *s);

/*
 * Load a sprite palette image. The image is chopped up into
 * sprites, based on 'sw' and 'sh', and added as new frames
 * in the sprite bank. Default values:
 *	Hot-spot:		(sw/2, sh/2)
 *	Collision radius:	0.2 * (sw + sh)
 *
 * Passing 0 for 'sw' and/or 'sh' makes pig_sprites() take
 * the respective value from the image width and/or height.
 *
 * Returns the index of the first frame loaded.
 */
int pig_sprites(PIG_engine *pe, const char *filename, int sw, int sh);

/* Set hot-spot of sprite 'frame' to (hotx, hoty) */
int pig_hotspot(PIG_engine *pe, int frame, int hotx, int hoty);

/* Set sprite/sprite collision zone radius of 'frame' */
int pig_radius(PIG_engine *pe, int frame, int radius);

void run_logic(PIG_engine *pe,Organism* neural_pig);

/*
 * Manually add a dirtyrect for pig_refresh().
 * 'dr' can be outside the engine viewport.
 */
void pig_dirty(PIG_engine *pe, SDL_Rect *dr);

void tile_area(PIG_engine *pe, SDL_Rect *r);

/*
 * Do what's needed to deal with the dirtyrects
 * and then make the new frame visible.
 */
void pig_flip(PIG_engine *pe);

/*
 * Refresh the viewport and any additional dirtyrects.
 *
 * Note that this does not refresh the entire viewport;
 * only the areas that have actually changed!
 */
void pig_refresh(PIG_engine *pe);

/*
 * Refresh the whole viewport, including sprites.
 */
void pig_refresh_all(PIG_engine *pe);

void show_rects(PIG_engine *pe, Dirtytable *dt);

/* Render a sprite "manually", bypassing the engine */
void pig_draw_sprite(PIG_engine *pe, int frame, int x, int y);

/*
 * Get the collision flags for the tile at (x, y),
 * where the unit of x and y is pixels. The return
 * is the PIG_sides flags for the tile, or PIG_NONE
 * if (x, y) is outside the map.
 */
int pig_test_map(PIG_engine *pe, int x, int y);

/*
 * Find the first "collidable" tile side when going from
 * (x1, y1) to (x2, y2). 'mask' determines which tile sides
 * are considered for collisions.
 *
 * Returns the side(s) hit, if any tile was hit. If the return
 * is non-zero, the PIG_cinfo struct at 'ci' contains detailed
 * information about the collision.
 */
int pig_test_map_vector(PIG_engine *pe,int x1,int y1,int x2,int y2,int mask,PIG_cinfo *ci);

/*
 * Map
 */
PIG_map *pig_map_open(PIG_engine *pe, int w, int h);
void pig_map_close(PIG_map *pm);

/* Load a tile palette image */
int pig_map_tiles(PIG_map *pm, const char *filename, int tw, int th);

/*
 * Set tile collision info for 'count' tiles, starting at
 * 'first'. Each tile in the tile palette has a set of
 * PIG_sides flags that determine which sides the tile are
 * considered for sprite/map collisions.
 */
void pig_map_collisions(PIG_map *pm, int first, int count,PIG_sides sides);

/*
 * Load a map from a string (one byte/tile). 'trans'
 * is a string used for translating 'data' into integer
 * tile indices. Each position in 'trans' corresponds
 * to one tile in the tile palette.
 */
int pig_map_from_string(PIG_map *pm, const char *trans, const char *data);

/*
 * Object
 */

PIG_object *get_object(PIG_engine *pe);

void free_object(PIG_object *po);

/*
 * Create an object with the initial position (x, y). If
 * 'last' is 1, the object will end up last in the
 * processing and rendering order, otherwise, first.
 *
 * Note that relative processing order is very important
 * to objects that chase each other and stuff like that!
 * If they're placed in the "wrong" order, the tracking
 * objects get an extra frame of reaction time, which is
 * annoying if it's not what you intend.
 */
PIG_object *pig_object_open(PIG_engine *pe, int x, int y, int last);

void close_object(PIG_object *po);

/*
 * Delete an object.
 *
 * Note that objects are never actually deleted. Instead,
 * they are placed in a free pool, where pig_object_open()
 * looks for objects to recycle.
 *
 * In fact, they are not even freed when you ask for it,
 * but rather kept around until the next rendered frame,
 * so they can be removed from the screen correctly.
 */
void pig_object_close(PIG_object *po);

/*
 * Close all objects.
 */
void pig_object_close_all(PIG_engine *pe);

/*
 * Find object by 'id', starting at object 'start'.
 *
 * The search starts at 'start' and is done in both
 * directions in parallel, assuming that the matching
 * object is near 'start' in the list. (It usually is
 * when dealing with linked objects.)
 *
 * Returns NULL if the object was not found.
 */
PIG_object *pig_object_find(PIG_object *start, int id);

#endif
