#ifndef _PigUtiles
#define _PigUtiles

/* Size of sprite frame table */
#define	PIG_MAX_SPRITES		1024


/* Graphics defines */
#define	SCREEN_W	800
#define	SCREEN_H	600
#define	TILE_W		32
#define	TILE_H		32
#define	MAP_W		25
#define	MAP_H		17
#define	FONT_SPACING	45
#define	PIG_FRAMES	12

/* World/physics constants */
#define	GRAV_ACC	4
#define	JUMP_SPEED	28

/* Sprite collision groups */
#define	GROUP_ENEMY	0x0001

/* Interpolated point */
struct PIG_ipoint {
	/* From the last logic frame: */
	double	ox, oy;		/* Position */

	/* From the last/current rendered frame: */
	int	gimage;		/* Sprite frame index */
	double	gx, gy;		/* Interpolated position */
};

/*
 * Game logic events
 *
 * PREFRAME:
 *	Occurs once per logic frame, before collision and
 *	off-screen detection, and before timer handlers.
 *
 * TIMERx:
 *	Occurs whenever timer x expires. Timers are one-
 *	shot, but can be reloaded by the handler for
 *	periodic action. Timer events are handled
 *	before collision and off-screen detection.
 *
 * HIT_TILE:
 *	Occurs when the hot-spot of an object hits a
 *	marked side of a tile, and the corresponding bit
 *	in 'tilemask' is set.
 *
 * HIT_OBJECT:
 *	Occurs when the collision circle of an object
 *	intersects the collision circle of another object,
 *	provided one or more bits in 'hitgroup' of the
 *	other object matches bits in 'hitmask'.
 *
 * OFFSCREEN:
 *	Occurs when an object is off-screen. This takes
 *	in account the hot-spot and bounding rectangle of
 *	the current sprite frame.
 *
 * POSTFRAME:
 *	Occurs once per logic frame, after collision
 *	detection, off-screen detection and all other
 *	events.
 *
 */
#define	PIG_TIMERS	3
typedef int PIG_events;
static const int PIG_PREFRAME=0,
				PIG_TIMER0=1,
				PIG_TIMER1=2,
				PIG_TIMER2=3,
				PIG_HIT_TILE=4,
				PIG_HIT_OBJECT=5,
				PIG_OFFSCREEN=6,
				PIG_POSTFRAME=7;

/* sides: */
typedef int PIG_sides;
static const int PIG_NONE=0,
				/* Bit positions */
				PIG_TOP_B=0,
				PIG_BOTTOM_B=1,
				PIG_LEFT_B=2,
				PIG_RIGHT_B=3;
/* Masks */
static const int PIG_TOP=1<<PIG_TOP_B,
				PIG_BOTTOM=1<<PIG_BOTTOM_B,
				PIG_LEFT=1<<PIG_LEFT_B,
				PIG_RIGHT=1<<PIG_RIGHT_B;
/* Combined masks */
static const int PIG_TL=PIG_TOP | PIG_LEFT,
				PIG_TR=PIG_TOP | PIG_RIGHT,
				PIG_BL=PIG_BOTTOM | PIG_LEFT,
				PIG_BR=PIG_BOTTOM | PIG_RIGHT,
				PIG_ALL=0xf;

typedef enum {
	PIG_UNCHANGED =	-10000000,
	PIG_MIN =	-10000001,
	PIG_CENTER =	-10000002,
	PIG_MAX =	-10000003
} PIG_values;

/* Collision info */
struct PIG_cinfo {
	double		ff;	/* Fractional frame */
	int		x, y;	/* Exact position */
	PIG_sides	sides;	/* Side of tile hit */
};

struct PIG_engine;
struct PIG_event;

/* Logic object */
struct PIG_object {
	PIG_engine	*owner;
	PIG_object	*next, *prev;

	int		id;		/* Unique ID. 0 means "free". */
	int rank; /* useful only for enemies, to get them always in the same order */

	int		ibase;		/* Sprite frame base index */
	int		image;		/* Sprite frame offset */
	double		x, y;		/* Position */
	double		vx, vy;		/* Speed */
	double		ax, ay;		/* Acceleration */
	PIG_ipoint	ip;
	int		tilemask;	/* Sprite/tile mask [PIG_ALL] */

	int		hitmask;	/* Sprite/sprite mask [0] */
	int		hitgroup;	/* Sprite/sprite group [0] */

	int		timer[PIG_TIMERS];	/* Down-counting timers */
	int		age;		/* Age timer (logic frames) */

	int		score;
	int		power;
	int		target;
	int		state;

	void (*handler)(PIG_object *po, const PIG_event *ev);

	void		*userdata;
	int type; // tells if this is the pig object, or enemy
};

struct PIG_event {
	PIG_events	type;

	/* For HIT_TILE, HIT_OBJECT and OFFSCREEN: */
	PIG_cinfo	cinfo;	/* Detailed collision info */

	/* For HIT_OBJECT: */
	PIG_object	*obj;	/* Which object? */
};

/* Level map */
struct PIG_map {
	PIG_engine	*owner;

	int		w, h;		/* Size of map (tiles) */
	unsigned char	*map;		/* 2D array of tile indices */
	unsigned char	*hit;		/* 2D array of collision flags */

	int		tw, th;		/* Size of one tile (pixels) */
	SDL_Surface	*tiles;		/* Tile palette image */
	unsigned char	hitinfo[256];	/* Collision info for the tiles */
};

/* Sprite frame */
struct PIG_sprite {
	int		w, h;		/* Size of sprite (pixels) */
	int		hotx, hoty;	/* Hot-spot offset (pixels) */
	int		radius;		/* Collision zone radius (pixels) */
	SDL_Surface	*surface;
};

/* Engine */
struct PIG_engine {
	/* Video stuff */
	SDL_Surface	*screen;
	SDL_Surface	*buffer;	/* For h/w surface displays */
	SDL_Surface	*surface;	/* Where to render to */
	int		pages;		/* # of display VRAM buffers */
	SDL_Rect	view;		/* Viewport pos & size (pixels) */
	int		page;		/* Current page (double buffer) */
	Dirtytable	*pagedirty[2];	/* One table for each page */
	Dirtytable	*workdirty;	/* The work dirtytable */

	/* "Live" switches */
	int		interpolation;
	int		direct;		/* 1 ==> render directly to screen */
	int		show_dirtyrects;

	/* Time */
	double		time;		/* Logic time (frames) */
	int		frame;		/* Logic time; integer part */

	/* Background graphics */
	PIG_map		*map;

	/* Sprites and stuff */
	PIG_object	*objects;
	PIG_object	*object_pool;
	int		object_id_counter;
	int		nsprites;
	PIG_sprite	**sprites;

	/* Logic frame global handlers */
	void (*before_objects)(PIG_engine *pe);
	void (*after_objects)(PIG_engine *pe);

	/* Space for user data */
	void		*userdata;
};


struct GAMESTATE {
	/* I/O */
	PIG_engine	*pe;
	Uint8		*keys;
	int		nice;
	int		refresh_screen;
	int		jump;

	/* Sprites */
	int		lifepig;
	int		scorefont;
	int		glassfont;
	int		icons;
	int		stars;
	int		pigframes;
	int		evil;
	int		slime;

	/* Global game state */
	int		running;
	int		level;
	int		lives;
	int		score;
	double	dashboard_time;
	int		enemycount;
	int		messages;

	/* Objects */
	PIG_object	*player;
};

typedef enum
{
	WAITING,
	WALKING,
	FALLING,
	KNOCKED,
	NEXT_LEVEL,
	DEAD
} OBJECT_states;

#endif
